# -*- encoding: utf-8 -*-
#
# The purpose of this collector is to compute the eigenvector for the input
# file containing a list of sites.
#
#
# :authors: Arturo Filastò
# :licence: see LICENSE

from ooni.kit import domclass
from ooni.templates import httpt

class DOMClassCollector(httpt.HTTPTest):
    name = "DOM class collector"
    author = "Arturo Filastò"
    version = 0.1

    followRedirects = True

    inputFile = ['file', 'f', None, 'The list of urls to build a domclass for']
    requiresTor = False
    requiresRoot = False

    def test_collect(self):
        if self.input:
            url = self.input
            return self.doRequest(url)
        else:
            raise Exception("No input specified")

    def processResponseBody(self, body):
        eigenvalues = domclass.compute_eigenvalues_from_DOM(content=body)
        self.report['eigenvalues'] = eigenvalues.tolist()



from datetime import datetime

def prettyDateNow():
    """
    Returns a good looking string for the local time.
    """
    return datetime.now().ctime()

def prettyDateNowUTC():
    """
    Returns a good looking string for utc time.
    """
    return datetime.utcnow().ctime()

def timestampNowLongUTC():
    """
    Returns a timestamp in the format of %Y-%m-%d %H:%M:%S in Universal Time
    Coordinates.
    """
    return datetime.utcnow().strftime("%Y-%m-%d %H:%M:%S")



import uuid
import yaml
import os

from copy import deepcopy

from datetime import datetime
from contextlib import contextmanager

from yaml.representer import SafeRepresenter
from yaml.emitter import Emitter
from yaml.serializer import Serializer
from yaml.resolver import Resolver

from twisted.python.util import untilConcludes
from twisted.internet import defer
from twisted.internet.error import ConnectionRefusedError

from ooni.utils import log
from ooni.tasks import Measurement
try:
    from scapy.packet import Packet
except ImportError:
    log.err("Scapy is not installed.")

    class Packet(object):
        pass

from ooni import errors

from ooni import otime
from ooni.utils import generate_filename

from ooni.settings import config

from ooni.tasks import ReportEntry


def createPacketReport(packet_list):
    """
    Takes as input a packet a list.

    Returns a dict containing a dict with the packet
    summary and the raw packet.
    """
    report = []
    for packet in packet_list:
        report.append({'raw_packet': str(packet),
                       'summary': str([packet])})
    return report


class OSafeRepresenter(SafeRepresenter):

    """
    This is a custom YAML representer that allows us to represent reports
    safely.
    It extends the SafeRepresenter to be able to also represent complex
    numbers and scapy packet.
    """

    def represent_data(self, data):
        """
        This is very hackish. There is for sure a better way either by using
        the add_multi_representer or add_representer, the issue though lies in
        the fact that Scapy packets are metaclasses that leads to
        yaml.representer.get_classobj_bases to not be able to properly get the
        base of class of a Scapy packet.
        XXX fully debug this problem
        """
        if isinstance(data, Packet):
            data = createPacketReport(data)
        return SafeRepresenter.represent_data(self, data)

    def represent_complex(self, data):
        if data.imag == 0.0:
            data = u'%r' % data.real
        elif data.real == 0.0:
            data = u'%rj' % data.imag
        elif data.imag > 0:
            data = u'%r+%rj' % (data.real, data.imag)
        else:
            data = u'%r%rj' % (data.real, data.imag)
        return self.represent_scalar(u'tag:yaml.org,2002:python/complex', data)

OSafeRepresenter.add_representer(complex,
                                 OSafeRepresenter.represent_complex)


class OSafeDumper(Emitter, Serializer, OSafeRepresenter, Resolver):

    """
    This is a modification of the YAML Safe Dumper to use our own Safe
    Representer that supports complex numbers.
    """

    def __init__(self, stream,
                 default_style=None, default_flow_style=None,
                 canonical=None, indent=None, width=None,
                 allow_unicode=None, line_break=None,
                 encoding=None, explicit_start=None, explicit_end=None,
                 version=None, tags=None):

        Emitter.__init__(self, stream, canonical=canonical,
                         indent=indent, width=width,
                         allow_unicode=allow_unicode, line_break=line_break)
        Serializer.__init__(self, encoding=encoding,
                            explicit_start=explicit_start,
                            explicit_end=explicit_end,
                            version=version, tags=tags)
        OSafeRepresenter.__init__(self, default_style=default_style,
                                  default_flow_style=default_flow_style)
        Resolver.__init__(self)


def safe_dump(data, stream=None, **kw):
    """
    Safely dump to a yaml file the specified data.
    """
    return yaml.dump_all([data], stream, Dumper=OSafeDumper, **kw)


class OReporter(object):

    def __init__(self, test_details):
        self.testDetails = test_details

    def createReport(self):
        """
        Override this with your own logic to implement tests.
        """
        raise NotImplemented

    def writeReportEntry(self, entry):
        """
        Takes as input an entry and writes a report for it.
        """
        raise NotImplemented

    def finish(self):
        pass


class YAMLReporter(OReporter):

    """
    These are useful functions for reporting to YAML format.

    report_destination:
        the destination directory of the report

    """

    def __init__(self, test_details, report_filename):
        self.report_path = report_filename
        OReporter.__init__(self, test_details)

    def _writeln(self, line):
        self._write("%s\n" % line)

    def _write(self, format_string, *args):
        if not self._stream:
            raise errors.ReportNotCreated
        if self._stream.closed:
            raise errors.ReportAlreadyClosed
        s = str(format_string)
        assert isinstance(s, type(''))
        if args:
            self._stream.write(s % args)
        else:
            self._stream.write(s)
        untilConcludes(self._stream.flush)

    def writeReportEntry(self, entry):
        log.debug("Writing report with YAML reporter")
        content = '---\n'
        if isinstance(entry, Measurement):
            report_entry = deepcopy(entry.testInstance.report)
        elif isinstance(entry, dict):
            report_entry = deepcopy(entry)
        else:
            raise Exception("Failed to serialise entry")
        content += safe_dump(report_entry)
        content += '...\n'
        self._write(content)

    def createReport(self):
        """
        Writes the report header and fire callbacks on self.created
        """
        log.debug("Creating %s" % self.report_path)
        self._stream = open(self.report_path, 'w+')

        self._writeln("###########################################")

        self._writeln("# OONI Probe Report for %s (%s)" % (
            self.testDetails['test_name'],
            self.testDetails['test_version'])
        )

        self._writeln("# %s" % otime.prettyDateNow())
        self._writeln("###########################################")

        self.writeReportEntry(self.testDetails)

    def finish(self):
        self._stream.close()


class OONIBReporter(OReporter):

    def __init__(self, test_details, collector_client):
        self.collector_client = collector_client

        self.reportId = None
        self.supportedFormats = ["yaml"]
        OReporter.__init__(self, test_details)

    def serializeEntry(self, entry, serialisation_format="yaml"):
        if serialisation_format == "json":
            if isinstance(entry, Measurement):
                report_entry = {
                    'input': entry.testInstance.report.pop('input', None),
                    'id': str(uuid.uuid4()),
                    'test_start_time': entry.testInstance.report.pop('test_start_time', None),
                    'measurement_start_time': entry.testInstance.report.pop('measurement_start_time', None),
                    'test_runtime': entry.testInstance.report.pop('test_runtime', None),
                    'test_keys': entry.testInstance.report
                }
            elif isinstance(entry, dict):
                report_entry = {
                    'input': entry.pop('input', None),
                    'id': str(uuid.uuid4()),
                    'test_start_time': entry.pop('test_start_time', None),
                    'measurement_start_time': entry.pop('measurement_start_time', None),
                    'test_runtime': entry.pop('test_runtime', None),
                    'test_keys': entry
                }
            else:
                raise Exception("Failed to serialise entry")
            report_entry.update(self.testDetails)
            return report_entry
        else:
            content = '---\n'
            if isinstance(entry, Measurement):
                report_entry = entry.testInstance.report
            elif isinstance(entry, dict):
                report_entry = entry
            else:
                raise Exception("Failed to serialise entry")
            content += safe_dump(report_entry)
            content += '...\n'
            return content

    @defer.inlineCallbacks
    def writeReportEntry(self, entry):
        if "json" in self.supportedFormats:
            serialization_format = 'json'
        else:
            serialization_format = 'yaml'

        log.debug("Updating report with id %s" % (self.reportId))
        entry_content = self.serializeEntry(entry, serialization_format)
        try:
            yield self.collector_client.updateReport(self.reportId,
                                                     serialization_format,
                                                     entry_content)
        except Exception as exc:
            log.err("Error in writing report entry")
            log.exception(exc)
            raise errors.OONIBReportUpdateError

    @defer.inlineCallbacks
    def createReport(self):
        """
        Creates a report on the oonib collector.
        """
        log.msg("Creating report with OONIB Reporter. Please be patient.")
        log.msg("This may take up to 1-2 minutes...")

        try:
            response = yield self.collector_client.createReport(
                                            self.testDetails
            )
        except ConnectionRefusedError:
            log.err("Connection to reporting backend failed "
                    "(ConnectionRefusedError)")
            raise errors.OONIBReportCreationError
        except errors.HostUnreachable:
            log.err("Host is not reachable (HostUnreachable error")
            raise errors.OONIBReportCreationError
        except (errors.OONIBInvalidInputHash,
                errors.OONIBInvalidNettestName):
            log.err("The specified input or nettests cannot be submitted to "
                    "this collector.")
            log.msg("Try running a different test or try reporting to a "
                    "different collector.")
            raise errors.OONIBReportCreationError
        except Exception, e:
            log.err("Failed to connect to reporter backend")
            log.exception(e)
            raise errors.OONIBReportCreationError

        self.reportId = response['report_id'].encode('ascii')
        self.backendVersion = response['backend_version']

        self.supportedFormats = response.get('supported_formats', ["yaml"])

        log.debug("Created report with id %s" % response['report_id'])
        defer.returnValue(response['report_id'])

    def finish(self):
        log.debug("Closing report with id %s" % self.reportId)
        return self.collector_client.closeReport(self.reportId)

class OONIBReportLog(object):

    """
    Used to keep track of report creation on a collector backend.
    """

    def __init__(self, file_name=None):
        if file_name is None:
            file_name = config.report_log_file
        self.file_name = file_name
        self.create_report_log()

    def get_report_log(self):
        with open(self.file_name) as f:
            report_log = yaml.safe_load(f)
        if not report_log:
            report_log = {}  # consumers expect dictionary structure
        return report_log

    @property
    def reports_incomplete(self):
        reports = []
        report_log = self.get_report_log()
        for report_file, value in report_log.items():
            if value['status'] in ('created'):
                try:
                    os.kill(value['pid'], 0)
                except:
                    reports.append((report_file, value))
            elif value['status'] in ('incomplete'):
                reports.append((report_file, value))
        return reports

    @property
    def reports_in_progress(self):
        reports = []
        report_log = self.get_report_log()
        for report_file, value in report_log.items():
            if value['status'] in ('created'):
                try:
                    os.kill(value['pid'], 0)
                    reports.append((report_file, value))
                except:
                    pass
        return reports

    @property
    def reports_to_upload(self):
        reports = []
        report_log = self.get_report_log()
        for report_file, value in report_log.items():
            if value['status'] in ('creation-failed', 'not-created'):
                reports.append((report_file, value))
        return reports

    def run(self, f, *arg, **kw):
        lock = defer.DeferredFilesystemLock(self.file_name + '.lock')
        d = lock.deferUntilLocked()

        def unlockAndReturn(r):
            lock.unlock()
            return r

        def execute(_):
            d = defer.maybeDeferred(f, *arg, **kw)
            d.addBoth(unlockAndReturn)
            return d

        d.addCallback(execute)
        return d

    def create_report_log(self):
        if not os.path.exists(self.file_name):
            with open(self.file_name, 'w+') as f:
                f.write(yaml.safe_dump({}))

    @contextmanager
    def edit_log(self):
        with open(self.file_name) as rfp:
            report = yaml.safe_load(rfp)
        # This should never happen.
        if report is None:
            report = {}
        with open(self.file_name, 'w+') as wfp:
            try:
                yield report
            finally:
                wfp.write(yaml.safe_dump(report))

    def _not_created(self, report_file):
        with self.edit_log() as report:
            report[report_file] = {
                'pid': os.getpid(),
                'created_at': datetime.now(),
                'status': 'not-created',
                'collector': None
            }

    def not_created(self, report_file):
        return self.run(self._not_created, report_file)

    def _created(self, report_file, collector_settings, report_id):
        with self.edit_log() as report:
            assert report_file is not None
            report[report_file] = {
                'pid': os.getpid(),
                'created_at': datetime.now(),
                'status': 'created',
                'collector': collector_settings,
                'report_id': report_id
            }
        return report_id

    def created(self, report_file, collector_settings, report_id):
        return self.run(self._created, report_file,
                        collector_settings, report_id)

    def _creation_failed(self, report_file, collector_settings):
        with self.edit_log() as report:
            report[report_file] = {
                'pid': os.getpid(),
                'created_at': datetime.now(),
                'status': 'creation-failed',
                'collector': collector_settings
            }

    def creation_failed(self, report_file, collector_settings):
        return self.run(self._creation_failed, report_file,
                        collector_settings)

    def _incomplete(self, report_file):
        with self.edit_log() as report:
            if report[report_file]['status'] != "created":
                raise errors.ReportNotCreated()
            report[report_file]['status'] = 'incomplete'

    def incomplete(self, report_file):
        return self.run(self._incomplete, report_file)

    def _closed(self, report_file):
        with self.edit_log() as report:
            rs = report[report_file]['status']
            if  rs != "created" and rs != "incomplete":
                raise errors.ReportNotCreated()
            del report[report_file]

    def closed(self, report_file):
        return self.run(self._closed, report_file)


class Report(object):
    reportId = None

    def __init__(self, test_details, report_filename,
                 reportEntryManager, collector_client=None,
                 no_yamloo=False):
        """
        This is an abstraction layer on top of all the configured reporters.

        It allows to lazily write to the reporters that are to be used.

        Args:

            test_details:
                A dictionary containing the test details.

            report_filename:
                The file path for the report to be written.

            reportEntryManager:
                an instance of :class:ooni.tasks.ReportEntryManager

            collector:
                The address of the oonib collector for this report.

            no_yamloo:
                If we should disable reporting to disk.
        """
        self.test_details = test_details
        self.collector_client = collector_client
        if report_filename is None:
            report_filename = self.generateReportFilename()
        self.report_filename = report_filename

        self.report_log = OONIBReportLog()

        self.yaml_reporter = None
        self.oonib_reporter = None
        self.no_yamloo = no_yamloo

        self.done = defer.Deferred()
        self.reportEntryManager = reportEntryManager

    def generateReportFilename(self):
        report_filename = generate_filename(self.test_details,
                                            prefix='report',
                                            extension='yamloo')
        report_path = os.path.join('.', report_filename)
        return os.path.abspath(report_path)

    def open_oonib_reporter(self):
        def creation_failed(failure):
            self.oonib_reporter = None
            return self.report_log.creation_failed(self.report_filename,
                                                   self.collector_client.settings)

        def created(report_id):
            if not self.oonib_reporter:
                return
            self.test_details['report_id'] = report_id
            return self.report_log.created(self.report_filename,
                                           self.collector_client.settings,
                                           report_id)

        d = self.oonib_reporter.createReport()
        d.addErrback(creation_failed)
        d.addCallback(created)
        return d

    @defer.inlineCallbacks
    def open(self):
        """
        This will create all the reports that need to be created and fires the
        created callback of the reporter whose report got created.
        """
        if self.collector_client:
            self.oonib_reporter = OONIBReporter(self.test_details,
                                                self.collector_client)
            self.test_details['report_id'] = yield self.open_oonib_reporter()

        if not self.no_yamloo:
            self.yaml_reporter = YAMLReporter(self.test_details,
                                              self.report_filename)
            if not self.oonib_reporter:
                yield self.report_log.not_created(self.report_filename)
            yield defer.maybeDeferred(self.yaml_reporter.createReport)

        defer.returnValue(self.reportId)

    def write(self, measurement):
        """
        Will return a deferred that will fire once the report for the specified
        measurement have been written to all the reporters.

        Args:

            measurement:
                an instance of :class:ooni.tasks.Measurement

        Returns:
            a deferred that will fire once all the report entries have
            been written or errbacks when no more reporters
        """

        d = defer.Deferred()
        deferreds = []

        def yaml_report_failed(failure):
            d.errback(failure)

        def oonib_report_failed(failure):
            return self.report_log.incomplete(self.report_filename)

        def all_reports_written(_):
            if not d.called:
                d.callback(None)

        if self.yaml_reporter:
            write_yaml_report = ReportEntry(self.yaml_reporter, measurement)
            self.reportEntryManager.schedule(write_yaml_report)
            write_yaml_report.done.addErrback(yaml_report_failed)
            deferreds.append(write_yaml_report.done)

        if self.oonib_reporter:
            write_oonib_report = ReportEntry(self.oonib_reporter, measurement)
            self.reportEntryManager.schedule(write_oonib_report)
            write_oonib_report.done.addErrback(oonib_report_failed)
            deferreds.append(write_oonib_report.done)

        dl = defer.DeferredList(deferreds)
        dl.addCallback(all_reports_written)

        return d

    def close(self):
        """
        Close the report by calling it's finish method.

        Returns:
            a :class:twisted.internet.defer.DeferredList that will fire when
            all the reports have been closed.

        """
        d = defer.Deferred()
        deferreds = []

        def yaml_report_failed(failure):
            d.errback(failure)

        def oonib_report_closed(result):
            return self.report_log.closed(self.report_filename)

        def oonib_report_failed(result):
            log.exception(result)
            log.err("Failed to close oonib report.")

        def all_reports_closed(_):
            if not d.called:
                d.callback(None)

        if self.yaml_reporter:
            close_yaml = defer.maybeDeferred(self.yaml_reporter.finish)
            close_yaml.addErrback(yaml_report_failed)
            deferreds.append(close_yaml)

        if self.oonib_reporter:
            close_oonib = self.oonib_reporter.finish()
            close_oonib.addCallback(oonib_report_closed)
            close_oonib.addErrback(oonib_report_failed)
            deferreds.append(close_oonib)

        dl = defer.DeferredList(deferreds)
        dl.addCallback(all_reports_closed)

        return d



import os
import shutil
import socket

from twisted.internet import defer
from twisted.web import error

from ooni import errors as e
from ooni.settings import config
from ooni.backend_client import CollectorClient, BouncerClient
from ooni.backend_client import WebConnectivityClient
from ooni.tests.bases import ConfigTestCase

from mock import MagicMock

input_id = '37e60e13536f6afe47a830bfb6b371b5cf65da66d7ad65137344679b24fdccd1'
deck_id = 'd4ae40ecfb3c1b943748cce503ab8233efce7823f3e391058fc0f87829c644ed'


class TestEnd2EndBackendClient(ConfigTestCase):
    def setUp(self):
        super(TestEnd2EndBackendClient, self).setUp()
        host = '127.0.0.1'
        port = 8889
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        try:
            s.connect((host, port))
            s.shutdown(2)

            data_dir = '/tmp/testooni'
            config.advanced.data_dir = data_dir

            if os.path.exists(data_dir):
                shutil.rmtree(data_dir)
            os.mkdir(data_dir)
            os.mkdir(os.path.join(data_dir, 'inputs'))
            os.mkdir(os.path.join(data_dir, 'decks'))
        except Exception:
            self.skipTest("OONIB must be listening on port 8888 to run this test (tor_hidden_service: false)")
        self.collector_client = CollectorClient('http://' + host + ':' + str(port))

    @defer.inlineCallbacks
    def test_query(self):
        res = yield self.collector_client.queryBackend('GET', '/policy/input')
        self.assertTrue(isinstance(res, list))

    @defer.inlineCallbacks
    def test_get_input_list(self):
        input_list = yield self.collector_client.getInputList()
        self.assertTrue(isinstance(input_list, list))

    @defer.inlineCallbacks
    def test_get_input_descriptor(self):
        input_descriptor = yield self.collector_client.getInput(input_id)
        for key in ['name', 'description',
                    'version', 'author', 'date', 'id']:
            self.assertTrue(hasattr(input_descriptor, key))

    @defer.inlineCallbacks
    def test_download_input(self):
        yield self.collector_client.downloadInput(input_id)

    @defer.inlineCallbacks
    def test_get_deck_list(self):
        deck_list = yield self.collector_client.getDeckList()
        self.assertTrue(isinstance(deck_list, list))

    @defer.inlineCallbacks
    def test_get_deck_descriptor(self):
        deck_descriptor = yield self.collector_client.getDeck(deck_id)
        for key in ['name', 'description',
                    'version', 'author', 'date', 'id']:
            self.assertTrue(hasattr(deck_descriptor, key))

    @defer.inlineCallbacks
    def test_download_deck(self):
        yield self.collector_client.downloadDeck(deck_id)

    def test_lookup_invalid_helpers(self):
        bouncer_client = BouncerClient('http://127.0.0.1:8888')
        return self.failUnlessFailure(
            bouncer_client.lookupTestHelpers([
                'sdadsadsa', 'dns'
            ]), e.CouldNotFindTestHelper)

    @defer.inlineCallbacks
    def test_lookup_no_test_helpers(self):
        bouncer_client = BouncerClient('http://127.0.0.1:8888')
        required_helpers = []
        helpers = yield bouncer_client.lookupTestHelpers(required_helpers)
        self.assertTrue('default' in helpers.keys())

    @defer.inlineCallbacks
    def test_lookup_test_helpers(self):
        bouncer_client = BouncerClient('http://127.0.0.1:8888')
        required_helpers = [u'http-return-json-headers', u'dns']
        helpers = yield bouncer_client.lookupTestHelpers(required_helpers)
        self.assertEqual(set(helpers.keys()), set(required_helpers + [u'default']))
        self.assertTrue(helpers['http-return-json-headers']['address'].startswith('http'))
        self.assertTrue(int(helpers['dns']['address'].split('.')[0]))

    @defer.inlineCallbacks
    def test_input_descriptor_not_found(self):
        yield self.assertFailure(self.collector_client.queryBackend('GET',
                                                             '/input/' + 'a'*64), e.OONIBInputDescriptorNotFound)

    @defer.inlineCallbacks
    def test_http_errors(self):
        yield self.assertFailure(self.collector_client.queryBackend('PUT',
                                                     '/policy/input'), error.Error)

    @defer.inlineCallbacks
    def test_create_report(self):
        res = yield self.collector_client.queryBackend('POST', '/report', {
            'software_name': 'spam',
            'software_version': '2.0',
            'probe_asn': 'AS0',
            'probe_cc': 'ZZ',
            'test_name': 'foobar',
            'test_version': '1.0',
            'input_hashes': []
        })
        assert isinstance(res['report_id'], unicode)

    @defer.inlineCallbacks
    def test_report_lifecycle(self):
        res = yield self.collector_client.queryBackend('POST', '/report', {
            'software_name': 'spam',
            'software_version': '2.0',
            'probe_asn': 'AS0',
            'probe_cc': 'ZZ',
            'test_name': 'foobar',
            'test_version': '1.0',
            'input_hashes': []
        })
        report_id = str(res['report_id'])

        res = yield self.collector_client.queryBackend('POST', '/report/' + report_id, {
            'content': '---\nspam: ham\n...\n'
        })

        res = yield self.collector_client.queryBackend('POST', '/report/' + report_id, {
            'content': '---\nspam: ham\n...\n'
        })

        res = yield self.collector_client.queryBackend('POST', '/report/' + report_id +
                                        '/close')


class TestBackendClient(ConfigTestCase):
    @defer.inlineCallbacks
    def test_web_connectivity_client_is_reachable(self):
        wcc = WebConnectivityClient(
            'https://web-connectivity.th.ooni.io')
        wcc.queryBackend = MagicMock()
        wcc.queryBackend.return_value = defer.succeed({"status": "ok"})
        result = yield wcc.isReachable()
        self.assertEqual(result, True)

    @defer.inlineCallbacks
    def test_web_connectivity_client_is_not_reachable(self):
        wcc = WebConnectivityClient(
            'https://web-connectivity.th.ooni.io')
        wcc.queryBackend = MagicMock()
        wcc.queryBackend.return_value = defer.fail(Exception())
        result = yield wcc.isReachable()
        self.assertEqual(result, False)


    @defer.inlineCallbacks
    def test_web_connectivity_client_control(self):
        wcc = WebConnectivityClient(
            'https://web-connectivity.th.ooni.io')
        wcc.queryBackend = MagicMock()
        wcc.queryBackend.return_value = defer.succeed({})
        yield wcc.control("http://example.com/", ["127.0.0.1:8080",
                                                  "127.0.0.1:8082"])
        wcc.queryBackend.assert_called_with(
            'POST', '/',
            query={
                "http_request": "http://example.com/",
                "tcp_connect": ["127.0.0.1:8080", "127.0.0.1:8082"]
            })


    @defer.inlineCallbacks
    def test_bouncer_client_lookup_collector(self):
        bcc = BouncerClient('https://bouncer.ooni.io')
        bcc.queryBackend = MagicMock()
        bcc.queryBackend.return_value = defer.succeed({})
        yield bcc.lookupTestCollector(["foo"])
        bcc.queryBackend.assert_called_with("POST",
                                            "/bouncer/net-tests",
                                            query={'net-tests': ["foo"]})


    @defer.inlineCallbacks
    def test_bouncer_client_lookup_test_helpers(self):
        bcc = BouncerClient('https://bouncer.ooni.io')
        bcc.queryBackend = MagicMock()
        bcc.queryBackend.return_value = defer.succeed({'spam': 'ham'})
        yield bcc.lookupTestHelpers(["foo"])
        bcc.queryBackend.assert_called_with("POST",
                                            "/bouncer/test-helpers",
                                            query={'test-helpers': ["foo"]})


    def test_backend_client_validates_url(self):
        raised = False
        try:
            cc = CollectorClient(settings={
                "type": "onion", "address": "http://invalid.onion"
            })
        except Exception as exc:
            raised = True
            self.assertIsInstance(exc, e.InvalidAddress)
        self.assertTrue(raised)



import os

from twisted.trial import unittest
from twisted.internet import defer, task

from ooni.managers import MeasurementManager

from ooni.tests.mocks import MockSuccessTask, MockFailTask, MockFailOnceTask, MockFailure
from ooni.tests.mocks import MockSuccessTaskWithTimeout, MockFailTaskThatTimesOut
from ooni.tests.mocks import MockTimeoutOnceTask, MockFailTaskWithTimeout
from ooni.tests.mocks import MockTaskManager, mockFailure, MockDirector
from ooni.tests.mocks import MockNetTest, MockSuccessMeasurement
from ooni.tests.mocks import MockFailMeasurement
from ooni.settings import config


class TestTaskManager(unittest.TestCase):
    timeout = 1

    def setUp(self):
        self.measurementManager = MockTaskManager()
        self.measurementManager.concurrency = 20
        self.measurementManager.retries = 2

        self.measurementManager.start()

        self.clock = task.Clock()
        data_dir = os.path.dirname(os.path.abspath(__file__))
        data_dir = os.path.join(data_dir, '..', '..', 'data')
        self.old_datadir = ""
        if hasattr(config.global_options, 'datadir'):
            self.old_datadir = config.global_options['datadir']
        config.global_options['datadir'] = data_dir
        config.set_paths()

    def tearDown(self):
        if self.old_datadir == "":
            del config.global_options['datadir']
        else:
            config.global_options['datadir'] = self.old_datadir
        config.set_paths()

    def schedule_successful_tasks(self, task_type, number=1):
        all_done = []
        for x in range(number):
            mock_task = task_type()
            all_done.append(mock_task.done)
            self.measurementManager.schedule(mock_task)

        d = defer.DeferredList(all_done)

        @d.addCallback
        def done(res):
            for task_result, task_instance in self.measurementManager.successes:
                self.assertEqual(task_result, 42)
                self.assertIsInstance(task_instance, task_type)

        return d

    def schedule_failing_tasks(self, task_type, number=1):
        all_done = []
        for x in range(number):
            mock_task = task_type()
            all_done.append(mock_task.done)
            mock_task.done.addErrback(lambda x: None)
            self.measurementManager.schedule(mock_task)

        d = defer.DeferredList(all_done)

        @d.addCallback
        def done(res):
            # 10*2 because 2 is the number of retries
            self.assertEqual(self.measurementManager.failures, number * 3)
            # XXX @aagbsn is there a reason why you switched to using an int
            # over a using a list?
            # self.assertEqual(len(self.measurementManager.failures), number*3)
            # for task_result, task_instance in self.measurementManager.failures:
            # self.assertEqual(task_result, mockFailure)
            # self.assertIsInstance(task_instance, task_type)

        return d

    def test_schedule_failing_with_mock_failure_task(self):
        mock_task = MockFailTask()
        self.measurementManager.schedule(mock_task)
        self.assertFailure(mock_task.done, MockFailure)
        return mock_task.done

    def test_schedule_successful_one_task(self):
        return self.schedule_successful_tasks(MockSuccessTask)

    def test_schedule_successful_one_task_with_timeout(self):
        return self.schedule_successful_tasks(MockSuccessTaskWithTimeout)

    def test_schedule_failing_tasks_that_timesout(self):
        self.measurementManager.retries = 0

        task_type = MockFailTaskThatTimesOut
        task_timeout = 5

        mock_task = task_type()
        mock_task.timeout = task_timeout
        mock_task.clock = self.clock

        self.measurementManager.schedule(mock_task)

        self.clock.advance(task_timeout)

        @mock_task.done.addBoth
        def done(res):
            self.assertEqual(self.measurementManager.failures, 1)
            # self.assertEqual(len(self.measurementManager.failures), 1)
            # for task_result, task_instance in self.measurementManager.failures:
            # self.assertIsInstance(task_instance, task_type)

        return mock_task.done

    def test_schedule_time_out_once(self):
        task_type = MockTimeoutOnceTask
        task_timeout = 5

        mock_task = task_type()
        mock_task.timeout = task_timeout
        mock_task.clock = self.clock

        self.measurementManager.schedule(mock_task)

        self.clock.advance(task_timeout)

        @mock_task.done.addBoth
        def done(res):
            self.assertEqual(self.measurementManager.failures, 1)
            # self.assertEqual(len(self.measurementManager.failures), 1)
            # for task_result, task_instance in self.measurementManager.failures:
            # self.assertIsInstance(task_instance, task_type)

            for task_result, task_instance in self.measurementManager.successes:
                self.assertEqual(task_result, 42)
                self.assertIsInstance(task_instance, task_type)

        return mock_task.done

    def test_schedule_failing_one_task(self):
        return self.schedule_failing_tasks(MockFailTask)

    def test_schedule_failing_one_task_with_timeout(self):
        return self.schedule_failing_tasks(MockFailTaskWithTimeout)

    def test_schedule_successful_ten_tasks(self):
        return self.schedule_successful_tasks(MockSuccessTask, number=10)

    def test_schedule_failing_ten_tasks(self):
        return self.schedule_failing_tasks(MockFailTask, number=10)

    def test_schedule_successful_27_tasks(self):
        return self.schedule_successful_tasks(MockSuccessTask, number=27)

    def test_schedule_failing_27_tasks(self):
        return self.schedule_failing_tasks(MockFailTask, number=27)

    def test_task_retry_and_succeed(self):
        mock_task = MockFailOnceTask()
        self.measurementManager.schedule(mock_task)

        @mock_task.done.addCallback
        def done(res):
            self.assertEqual(self.measurementManager.failures, 1)
            # self.assertEqual(len(self.measurementManager.failures), 1)
            # self.assertEqual(self.measurementManager.failures,
            # [(mockFailure, mock_task)])
            self.assertEqual(self.measurementManager.successes,
                             [(42, mock_task)])

        return mock_task.done

    def test_task_retry_and_succeed_56_tasks(self):
        """
        XXX this test fails in a non-deterministic manner.
        """
        all_done = []
        number = 56
        for x in range(number):
            mock_task = MockFailOnceTask()
            all_done.append(mock_task.done)
            self.measurementManager.schedule(mock_task)

        d = defer.DeferredList(all_done)

        @d.addCallback
        def done(res):
            self.assertEqual(self.measurementManager.failures, number)
            # self.assertEqual(len(self.measurementManager.failures), number)
            for task_result, task_instance in self.measurementManager.successes:
                self.assertEqual(task_result, 42)
                self.assertIsInstance(task_instance, MockFailOnceTask)

        return d


class TestMeasurementManager(unittest.TestCase):
    def setUp(self):
        mock_director = MockDirector()

        self.measurementManager = MeasurementManager()
        self.measurementManager.director = mock_director

        self.measurementManager.concurrency = 10
        self.measurementManager.retries = 2

        self.measurementManager.start()

        self.mockNetTest = MockNetTest()

    def test_schedule_and_net_test_notified(self, number=1):
        # XXX we should probably be inheriting from the base test class
        mock_task = MockSuccessMeasurement(self.mockNetTest)
        self.measurementManager.schedule(mock_task)

        @mock_task.done.addCallback
        def done(res):
            self.assertEqual(self.mockNetTest.successes,
                             [42])

            self.assertEqual(len(self.mockNetTest.successes), 1)

        return mock_task.done

    def test_schedule_failing_one_measurement(self):
        mock_task = MockFailMeasurement(self.mockNetTest)
        self.measurementManager.schedule(mock_task)

        @mock_task.done.addErrback
        def done(failure):
            self.assertEqual(self.measurementManager.failures, 3)
            # self.assertEqual(len(self.measurementManager.failures), 3)

            self.assertEqual(failure, mockFailure)
            self.assertEqual(len(self.mockNetTest.successes), 0)

        return mock_task.done



from __future__ import print_function

import os
import sys

from ooni.constants import CANONICAL_BOUNCER_ONION
from ooni.report import __version__
from ooni.report import tool
from ooni.settings import config

from twisted.python import usage


class Options(usage.Options):

    synopsis = """%s [options] upload | status
""" % (os.path.basename(sys.argv[0]),)

    optFlags = [
        ["default-collector", "d", "Upload the reports to the default "
                                   "collector that is looked up with the "
                                   "canonical bouncer."]
    ]

    optParameters = [
        ["configfile", "f", None,
         "Specify the configuration file to use."],
        ["collector", "c", None,
         "Specify the collector to upload the result to."],
        ["bouncer", "b", None,
         "Specify the bouncer to query for a collector."]
    ]

    def opt_version(self):
        print("oonireport version: %s" % __version__)
        sys.exit(0)

    def parseArgs(self, *args):
        if len(args) == 0:
            raise usage.UsageError(
                "Must specify at least one command"
            )
            return
        self['command'] = args[0]
        if self['command'] not in ("upload", "status"):
            raise usage.UsageError(
                "Must specify either command upload or status"
            )
        if self['command'] == "upload":
            try:
                self['report_file'] = args[1]
            except IndexError:
                self['report_file'] = None


def tor_check():
    if not config.tor.socks_port:
        print("Currently oonireport requires that you start Tor yourself "
              "and set the socks_port inside of ooniprobe.conf")
        sys.exit(1)


def run(args=sys.argv[1:]):
    options = Options()
    try:
        options.parseOptions(args)
    except Exception as exc:
        print("Error: %s" % exc)
        print(options)
        sys.exit(2)
    config.global_options = dict(options)
    config.set_paths()
    config.read_config_file()

    if options['default-collector']:
        options['bouncer'] = CANONICAL_BOUNCER_ONION

    if options['command'] == "upload" and options['report_file']:
        tor_check()
        return tool.upload(options['report_file'],
                           options['collector'],
                           options['bouncer'])
    elif options['command'] == "upload":
        tor_check()
        return tool.upload_all(options['collector'],
                               options['bouncer'])
    elif options['command'] == "status":
        return tool.status()
    else:
        print(options)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
  tls_handshake.py
  ----------------

  This file contains test cases for determining if a TLS handshake completes
  successfully, including ways to test if a TLS handshake which uses Mozilla
  Firefox's current ciphersuite list completes. Rather than using Twisted and
  OpenSSL's methods for automatically completing a handshake, which includes
  setting all the parameters, such as the ciphersuite list, these tests use
  non-blocking sockets and implement asychronous error-handling transversal of
  OpenSSL's memory BIO state machine, allowing us to determine where and why a
  handshake fails.

  This network test is a complete rewrite of a pseudonymously contributed
  script by Hackerberry Finn, in order to fit into OONI's core network tests.

  @authors: Isis Agora Lovecruft <isis@torproject.org>
  @license: see included LICENSE file
  @copyright: © 2013 Isis Lovecruft, The Tor Project Inc.
"""

from socket import error   as socket_error
from socket import timeout as socket_timeout
from socket import inet_aton as socket_inet_aton
from socket import gethostbyname as socket_gethostbyname
from time   import sleep

import os
import socket
import struct
import sys
import types

import ipaddr
import OpenSSL

from OpenSSL                import SSL, crypto
from twisted.internet       import defer, threads
from twisted.python         import usage, failure

from ooni       import nettest
from ooni.utils import log
from ooni.errors import InsufficientPrivileges
from ooni.settings import config

## For a way to obtain the current version of Firefox's default ciphersuite
## list, see https://trac.torproject.org/projects/tor/attachment/ticket/4744/
## and the attached file "get_mozilla_files.py".
##
## Note, however, that doing so requires the source code to the version of
## firefox that you wish to emulate.

firefox_ciphers = ["ECDHE-ECDSA-AES256-SHA",
                   "ECDHE-RSA-AES256-SHA",
                   "DHE-RSA-CAMELLIA256-SHA",
                   "DHE-DSS-CAMELLIA256-SHA",
                   "DHE-RSA-AES256-SHA",
                   "DHE-DSS-AES256-SHA",
                   "ECDH-ECDSA-AES256-CBC-SHA",
                   "ECDH-RSA-AES256-CBC-SHA",
                   "CAMELLIA256-SHA",
                   "AES256-SHA",
                   "ECDHE-ECDSA-RC4-SHA",
                   "ECDHE-ECDSA-AES128-SHA",
                   "ECDHE-RSA-RC4-SHA",
                   "ECDHE-RSA-AES128-SHA",
                   "DHE-RSA-CAMELLIA128-SHA",
                   "DHE-DSS-CAMELLIA128-SHA",]


class SSLContextError(usage.UsageError):
    """Raised when we're missing the SSL context method, or incompatible
    contexts were provided. The SSL context method should be one of the
    following:

        :attr:`OpenSSL.SSL.SSLv2_METHOD <OpenSSL.SSL.SSLv2_METHOD>`
        :attr:`OpenSSL.SSL.SSLv23_METHOD <OpenSSL.SSL.SSLv23_METHOD>`
        :attr:`OpenSSL.SSL.SSLv3_METHOD <OpenSSL.SSL.SSLv3_METHOD>`
        :attr:`OpenSSL.SSL.TLSv1_METHOD <OpenSSL.SSL.TLSv1_METHOD>`

    To use the pre-defined error messages, construct with one of the
    :meth:`SSLContextError.errors.keys <keys>` as the ``message`` string, like
    so:

        ``SSLContextError('NO_CONTEXT')``
    """

    #: Pre-defined error messages.
    errors = {
        'NO_CONTEXT': 'No SSL/TLS context chosen! Defaulting to TLSv1.',
        'INCOMPATIBLE': str("Testing TLSv1 (option '--tls1') is incompatible "
                            + "with testing SSL ('--ssl2' and '--ssl3')."),
        'MISSING_SSLV2': str("Your version of OpenSSL was compiled without "
                             + "support for SSLv2. This is normal on newer "
                             + "versions of OpenSSL, but it means that you "
                             + "will be unable to test SSLv2 handshakes "
                             + "without recompiling OpenSSL."), }

    def __init__(self, message):
        if message in self.errors.keys():
            message = self.errors[message]
        super(usage.UsageError, self).__init__(message)

class HostUnreachableError(Exception):
    """Raised when the host IP address appears to be unreachable."""
    pass

class HostUnresolveableError(Exception):
    """Raised when the host address appears to be unresolveable."""
    pass

class ConnectionTimeout(Exception):
    """Raised when we receive a :class:`socket.timeout <timeout>`, in order to
    pass the Exception along to
    :func:`TLSHandshakeTest.test_handshake.connectionFailed
    <connectionFailed>`.
    """
    pass

class HandshakeOptions(usage.Options):
    """ :class:`usage.Options <Options>` parser for the tls-handshake test."""
    optParameters = [
        ['host', 'h', None,
         'Remote host IP address (v4/v6) and port, i.e. "1.2.3.4:443"'],
        ['port', 'p', None,
         'Use this port for all hosts, regardless of port specified in file'],
        ['ciphersuite', 'c', None ,
         'File containing ciphersuite list, one per line'],]
    optFlags = [
        ['ssl2', '2', 'Use SSLv2'],
        ['ssl3', '3', 'Use SSLv3'],
        ['tls1', 't', 'Use TLSv1'],]

class HandshakeTest(nettest.NetTestCase):
    """An ooniprobe NetTestCase for determining if we can complete a TLS/SSL
    handshake with a remote host.
    """
    name         = 'tls-handshake'
    author       = 'Isis Lovecruft <isis@torproject.org>'
    description  = 'A test to determing if we can complete a TLS hankshake.'
    version      = '0.0.3'

    requiresRoot = False
    requiresTor  = False
    usageOptions = HandshakeOptions

    host = None
    inputFile = ['file', 'f', None, 'List of <HOST>:<PORT>s to test']

    #: Default SSL/TLS context method.
    context = SSL.Context(SSL.TLSv1_METHOD)

    def setUp(self, *args, **kwargs):
        """Set defaults for a :class:`HandshakeTest <HandshakeTest>`."""

        self.ciphers = list()

        if self.localOptions:
            options = self.localOptions

            ## check that we're testing an IP:PORT, else exit gracefully:
            if not (options['host']  or options['file']):
                raise SystemExit("Need --host or --file!")
            if options['host']:
                self.host = options['host']

            ## If no context was chosen, explain our default to the user:
            if not (options['ssl2'] or options['ssl3'] or options['tls1']):
                try: raise SSLContextError('NO_CONTEXT')
                except SSLContextError as sce: log.err(sce.message)
                context = None
            else:
                ## If incompatible contexts were chosen, inform the user:
                if options['tls1'] and (options['ssl2'] or options['ssl3']):
                    try: raise SSLContextError('INCOMPATIBLE')
                    except SSLContextError as sce: log.err(sce.message)
                    finally: log.msg('Defaulting to testing only TLSv1.')
                elif options['ssl2']:
                    try:
                        if not options['ssl3']:
                            context = SSL.Context(SSL.SSLv2_METHOD)
                        else:
                            context = SSL.Context(SSL.SSLv23_METHOD)
                    except ValueError as ve:
                        log.err(ve.message)
                        try: raise SSLContextError('MISSING_SSLV2')
                        except SSLContextError as sce:
                            log.err(sce.message)
                            log.msg("Falling back to testing only TLSv1.")
                            context = SSL.Context(SSL.TLSv1_METHOD)
                elif options['ssl3']:
                    context = SSL.Context(SSL.SSLv3_METHOD)
            ## finally, reset the context if the user's choice was okay:
            if context: self.context = context

            ## if we weren't given a file with a list of ciphersuites to use,
            ## then use the firefox default list:
            if not options['ciphersuite']:
                self.ciphers = firefox_ciphers
                log.msg('Using default Firefox ciphersuite list.')
            else:
                if os.path.isfile(options['ciphersuite']):
                    log.msg('Using ciphersuite list from "%s"'
                            % options['ciphersuite'])
                    with open(options['ciphersuite']) as cipherfile:
                        for line in cipherfile.readlines():
                            self.ciphers.append(line.strip())
            self.ciphersuite = ":".join(self.ciphers)

        if getattr(config.advanced, 'default_timeout', None) is not None:
            self.timeout = config.advanced.default_timeout
        else:
            self.timeout = 30   ## default the timeout to 30 seconds

        ## xxx For debugging, set the socket timeout higher anyway:
        self.timeout = 30

        ## We have to set the default timeout on our sockets before creation:
        socket.setdefaulttimeout(self.timeout)
    def isIP(self,addr):
        try:
            socket_inet_aton(addr)
            return True
        except socket_error:
            return False

    def resolveHost(self,addr):
        try:
            return socket_gethostbyname(addr)
        except socket_error:
            raise HostUnresolveableError

    def splitInput(self, input):
        addr, port = input.strip().rsplit(':', 1)

        #if addr is hostname it is resolved to ip
        if not self.isIP(addr):
            addr=self.resolveHost(addr)

        if self.localOptions['port']:
            port = self.localOptions['port']
        return (str(addr), int(port))

    def inputProcessor(self, file=None):
        if self.host:
            yield self.splitInput(self.host)
        if os.path.isfile(file):
            with open(file) as fh:
                for line in fh.readlines():
                    if line.startswith('#'):
                        continue
                    try:
                        yield self.splitInput(line)
                    except HostUnresolveableError:
                        continue

    def buildSocket(self, addr):
        global s
        ip = ipaddr.IPAddress(addr) ## learn if we're IPv4 or IPv6
        if ip.version == 4:
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        elif ip.version == 6:
            s = socket.socket(socket.AF_INET6, socket.SOCK_STREAM)
        return s

    def getContext(self):
        self.context.set_cipher_list(self.ciphersuite)
        return self.context

    @staticmethod
    def getPeerCert(connection, get_chain=False):
        """Get the PEM-encoded certificate or cert chain of the remote host.

        :param connection: A :class:`OpenSSL.SSL.Connection <Connection>`.
        :param bool get_chain: If True, get the all certificates in the
            chain. Otherwise, only get the remote host's certificate.
        :returns: A PEM-encoded x509 certificate. If
            :param:`getPeerCert.get_chain <get_chain>` is True, returns a list
            of PEM-encoded x509 certificates.
        """
        if not get_chain:
            x509_cert = connection.get_peer_certificate()
            pem_cert = crypto.dump_certificate(crypto.FILETYPE_PEM, x509_cert)
            return pem_cert
        else:
            cert_chain = []
            x509_cert_chain = connection.get_peer_cert_chain()
            for x509_cert in x509_cert_chain:
                pem_cert = crypto.dump_certificate(crypto.FILETYPE_PEM,
                                                   x509_cert)
                cert_chain.append(pem_cert)
            return cert_chain

    @staticmethod
    def getX509Name(certificate, get_components=False):
        """Get the DER-encoded form of the Name fields of an X509 certificate.

        @param certificate: A :class:`OpenSSL.crypto.X509Name` object.
        @param get_components: A boolean. If True, returns a list of tuples of
                               the (name, value)s of each Name field in the
                               :param:`certificate`. If False, returns the DER
                               encoded form of the Name fields of the
                               :param:`certificate`.
        """
        x509_name = None

        try:
            assert isinstance(certificate, crypto.X509Name), \
                "getX509Name takes OpenSSL.crypto.X509Name as first argument!"
            x509_name = crypto.X509Name(certificate)
        except AssertionError as ae:
            log.err(ae)
        except Exception as exc:
            log.exception(exc)

        if not x509_name is None:
            if not get_components:
                return x509_name.der()
            else:
                return x509_name.get_components()
        else:
            log.debug("getX509Name: got None for ivar x509_name")

    @staticmethod
    def getPublicKey(key):
        """Get the PEM-encoded format of a host certificate's public key.

        :param key: A :class:`OpenSSL.crypto.PKey <crypto.PKey>` object.
        """
        try:
            assert isinstance(key, crypto.PKey), \
                "getPublicKey expects type OpenSSL.crypto.PKey for parameter key"
        except AssertionError as ae:
            log.err(ae)
        else:
            pubkey = crypto.dump_privatekey(crypto.FILETYPE_PEM, key)
            return pubkey

    def test_handshake(self):
        """xxx fill me in"""

        def makeConnection(host):
            """Create a socket to the remote host's IP address, then get the
            TLS/SSL context method and ciphersuite list. Lastly, initiate a
            connection to the host.

            :param tuple host: A tuple of the remote host's IP address as a
                string, and an integer specifying the remote host port, i.e.
                ('1.1.1.1',443)
            :raises: :exc:`ConnectionTimeout` if the socket timed out.
            :returns: A :class:`OpenSSL.SSL.Connection <Connection>`.
            """
            addr, port = host
            sckt = self.buildSocket(addr)
            context = self.getContext()
            connection = SSL.Connection(context, sckt)
            try:
               connection.connect(host)
            except socket_timeout as stmo:
               error = ConnectionTimeout(stmo.message)
               return failure.Failure(error)
            else:
               return connection

        def connectionFailed(connection, host):
            """Handle errors raised while attempting to create the socket and
            :class:`OpenSSL.SSL.Connection <Connection>`, and setting the
            TLS/SSL context.

            :type connection: :exc:Exception
            :param connection: The exception that was raised in
                :func:`HandshakeTest.test_handshake.makeConnection
                <makeConnection>`.
            :param tuple host: A tuple of the host IP address as a string, and
                an int specifying the host port, i.e. ('1.1.1.1', 443)
            :rtype: :exc:Exception
            :returns: The original exception.
            """
            addr, port = host

            if not isinstance(connection, SSL.Connection):
                if isinstance(connection, IOError):
                    ## On some *nix distros, /dev/random is 0600 root:root and
                    ## we get a permissions error when trying to read
                    if connection.message.find("[Errno 13]"):
                        raise InsufficientPrivileges(
                            "%s" % connection.message.split("[Errno 13]", 1)[1])
                elif isinstance(connection, socket_error):
                    if connection.message.find("[Errno 101]"):
                        raise HostUnreachableError(
                            "Host unreachable: %s:%s" % (addr, port))
                elif isinstance(connection, Exception):
                    log.debug("connectionFailed: got Exception:")
                    log.err("Connection failed with reason: %s"
                            % connection.message)
                else:
                    log.err("Connection failed with reason: %s" % str(connection))

            self.report['host'] = addr
            self.report['port'] = port
            self.report['state'] = 'CONNECTION_FAILED'

            return connection

        def connectionSucceeded(connection, host, timeout):
            """If we have created a connection, set the socket options, and log
            the connection state and peer name.

            :param connection: A :class:`OpenSSL.SSL.Connection <Connection>`.
            :param tuple host: A tuple of the remote host's IP address as a
                string, and an integer specifying the remote host port, i.e.
                ('1.1.1.1',443)
            """

            ## xxx TODO to get this to work with a non-blocking socket, see how
            ##     twisted.internet.tcp.Client handles socket objects.
            connection.setblocking(1)

            ## Set the timeout on the connection:
            ##
            ## We want to set SO_RCVTIMEO and SO_SNDTIMEO, which both are
            ## defined in the socket option definitions in <sys/socket.h>, and
            ## which both take as their value, according to socket(7), a
            ## struct timeval, which is defined in the libc manual:
            ## https://www.gnu.org/software/libc/manual/html_node/Elapsed-Time.html
            timeval = struct.pack('ll', int(timeout), 0)
            connection.setsockopt(socket.SOL_SOCKET, socket.SO_RCVTIMEO, timeval)
            connection.setsockopt(socket.SOL_SOCKET, socket.SO_SNDTIMEO, timeval)

            ## Set the connection state to client mode:
            connection.set_connect_state()

            peer_name, peer_port = connection.getpeername()
            if peer_name:
                log.msg("Connected to %s" % peer_name)
            else:
                log.debug("Couldn't get peer name from connection: %s" % host)
                log.msg("Connected to %s" % host)
            log.debug("Connection state: %s " % connection.state_string())

            return connection

        def connectionRenegotiate(connection, host, error_message):
            """Handle a server-initiated SSL/TLS handshake renegotiation.

            :param connection: A :class:`OpenSSL.SSL.Connection <Connection>`.
            :param tuple host: A tuple of the remote host's IP address as a
                string, and an integer specifying the remote host port, i.e.
                ('1.1.1.1',443)
            """

            log.msg("Server requested renegotiation from: %s" % host)
            log.debug("Renegotiation reason: %s" % error_message)
            log.debug("State: %s" % connection.state_string())

            if connection.renegotiate():
                log.debug("Renegotiation possible.")
                log.msg("Retrying handshake with %s..." % host)
                try:
                    connection.do_handshake()
                    while connection.renegotiate_pending():
                        log.msg("Renegotiation with %s in progress..." % host)
                        log.debug("State: %s" % connection.state_string())
                        sleep(1)
                    else:
                        log.msg("Renegotiation with %s complete!" % host)
                except SSL.WantReadError, wre:
                    connection = handleWantRead(connection)
                    log.debug("State: %s" % connection.state_string())
                except SSL.WantWriteError, wwe:
                    connection = handleWantWrite(connection)
                    log.debug("State: %s" % connection.state_string())
            return connection

        def connectionShutdown(connection, host):
            """Handle shutting down a :class:`OpenSSL.SSL.Connection
            <Connection>`, including correct handling of halfway shutdown
            connections.

            Calls to :meth:`OpenSSL.SSL.Connection.shutdown
            <Connection.shutdown()>` return a boolean value -- if the
            connection is already shutdown, it returns True, else it returns
            false. Thus we loop through a block which detects if the connection
            is an a partial shutdown state and corrects that if that is the
            case, else it waits for one second, then attempts shutting down the
            connection again.

            Detection of a partial shutdown state is done through
            :meth:`OpenSSL.SSL.Connection.get_shutdown
            <Connection.get_shutdown()>` which queries OpenSSL for a bitvector
            of the server and client shutdown states. For example, the binary
            string '0b00' is an open connection, and '0b10' is a partially
            closed connection that has been shutdown on the serverside.

            :param connection: A :class:`OpenSSL.SSL.Connection <Connection>`.
            :param tuple host: A tuple of the remote host's IP address as a
                string, and an integer specifying the remote host port, i.e.
                ('1.1.1.1',443)
            """

            peername, peerport = host

            if isinstance(connection, SSL.Connection):
                log.msg("Closing connection to %s:%d..." % (peername, peerport))
                while not connection.shutdown():
                    ## if the connection is halfway shutdown, we have to
                    ## wait for a ZeroReturnError on connection.recv():
                    if (bin(connection.get_shutdown()) == '0b01') \
                            or (bin(connection.get_shutdown()) == '0b10'):
                        try:
                            _read_buffer = connection.pending()
                            connection.recv(_read_buffer)
                        except SSL.ZeroReturnError, zre: continue
                    else:
                        sleep(1)
                else:
                    log.msg("Closed connection to %s:%d"
                            % (peername, peerport))
            elif isinstance(connection, types.NoneType):
                log.debug("connectionShutdown: got NoneType for connection")
                return
            else:
                log.debug("connectionShutdown: expected connection, got %r"
                          % connection.__repr__())

            return connection

        def handleWantRead(connection):
            """From OpenSSL memory BIO documentation on ssl_read():

                If the underlying BIO is blocking, SSL_read() will only
                return, once the read operation has been finished or an error
                occurred, except when a renegotiation take place, in which
                case a SSL_ERROR_WANT_READ may occur. This behaviour can be
                controlled with the SSL_MODE_AUTO_RETRY flag of the
                SSL_CTX_set_mode(3) call.

                If the underlying BIO is non-blocking, SSL_read() will also
                return when the underlying BIO could not satisfy the needs of
                SSL_read() to continue the operation. In this case a call to
                SSL_get_error(3) with the return value of SSL_read() will
                yield SSL_ERROR_WANT_READ or SSL_ERROR_WANT_WRITE. As at any
                time a re-negotiation is possible, a call to SSL_read() can
                also cause write operations!  The calling process then must
                repeat the call after taking appropriate action to satisfy the
                needs of SSL_read(). The action depends on the underlying
                BIO. When using a non-blocking socket, nothing is to be done,
                but select() can be used to check for the required condition.

            And from the OpenSSL memory BIO documentation on ssl_get_error():

                SSL_ERROR_WANT_READ, SSL_ERROR_WANT_WRITE

                The operation did not complete; the same TLS/SSL I/O function
                should be called again later. If, by then, the underlying BIO
                has data available for reading (if the result code is
                SSL_ERROR_WANT_READ) or allows writing data
                (SSL_ERROR_WANT_WRITE), then some TLS/SSL protocol progress
                will take place, i.e. at least part of an TLS/SSL record will
                be read or written. Note that the retry may again lead to a
                SSL_ERROR_WANT_READ or SSL_ERROR_WANT_WRITE condition. There
                is no fixed upper limit for the number of iterations that may
                be necessary until progress becomes visible at application
                protocol level.

                For socket BIOs (e.g. when SSL_set_fd() was used), select() or
                poll() on the underlying socket can be used to find out when
                the TLS/SSL I/O function should be retried.

                Caveat: Any TLS/SSL I/O function can lead to either of
                SSL_ERROR_WANT_READ and SSL_ERROR_WANT_WRITE. In particular,
                SSL_read() or SSL_peek() may want to write data and
                SSL_write() may want to read data. This is mainly because
                TLS/SSL handshakes may occur at any time during the protocol
                (initiated by either the client or the server); SSL_read(),
                SSL_peek(), and SSL_write() will handle any pending
                handshakes.

            Also, see http://stackoverflow.com/q/3952104
            """
            try:
                while connection.want_read():
                    self.state = connection.state_string()
                    log.debug("Connection to %s HAS want_read" % host)
                    _read_buffer = connection.pending()
                    log.debug("Rereading %d bytes..." % _read_buffer)
                    sleep(1)
                    rereceived = connection.recv(int(_read_buffer))
                    log.debug("Received %d bytes" % rereceived)
                    log.debug("State: %s" % connection.state_string())
                else:
                    self.state = connection.state_string()
                    peername, peerport = connection.getpeername()
                    log.debug("Connection to %s:%s DOES NOT HAVE want_read"
                              % (peername, peerport))
                    log.debug("State: %s" % connection.state_string())
            except SSL.WantWriteError, wwe:
                self.state = connection.state_string()
                log.debug("Got WantWriteError while handling want_read")
                log.debug("WantWriteError: %s" % wwe.message)
                log.debug("Switching to handleWantWrite()...")
                handleWantWrite(connection)
            return connection

        def handleWantWrite(connection):
            """See :func:HandshakeTest.test_hanshake.handleWantRead """
            try:
                while connection.want_write():
                    self.state = connection.state_string()
                    log.debug("Connection to %s HAS want_write" % host)
                    sleep(1)
                    resent = connection.send("o\r\n")
                    log.debug("Sent: %d" % resent)
                    log.debug("State: %s" % connection.state_string())
            except SSL.WantReadError, wre:
                self.state = connection.state_string()
                log.debug("Got WantReadError while handling want_write")
                log.debug("WantReadError: %s" % wre.message)
                log.debug("Switching to handleWantRead()...")
                handleWantRead(connection)
            return connection

        def doHandshake(connection):
            """Attempt a TLS/SSL handshake with the host.

            If, after the first attempt at handshaking, OpenSSL's memory BIO
            state machine does not report success, then try reading and
            writing from the connection, and handle any SSL_ERROR_WANT_READ or
            SSL_ERROR_WANT_WRITE which occurs.

            If multiple want_reads occur, then try renegotiation with the
            host, and start over. If multiple want_writes occur, then it is
            possible that the connection has timed out, and move on to the
            connectionShutdown step.

            :param connection: A :class:`OpenSSL.SSL.Connection <Connection>`.
            :ivar peername: The host IP address, as reported by
                :meth:`Connection.getpeername <connection.getpeername()>`.
            :ivar peerport: The host port, reported by
                :meth:`Connection.getpeername <connection.getpeername()>`.
            :ivar int sent: The number of bytes sent to to the remote host.
            :ivar int received: The number of bytes received from the remote
                                host.
            :ivar int _read_buffer: The max bytes that can be read from the
                                    connection.
            :returns: The :param:`doHandshake.connection <connection>` with
                      handshake completed, else the unhandled error that was
                      raised.
            """
            peername, peerport = connection.getpeername()

            try:
                log.msg("Attempting handshake: %s" % peername)
                connection.do_handshake()
            except OpenSSL.SSL.WantReadError() as wre:
                self.state = connection.state_string()
                log.debug("Handshake state: %s" % self.state)
                log.debug("doHandshake: WantReadError on first handshake attempt.")
                connection = handleWantRead(connection)
            except OpenSSL.SSL.WantWriteError() as wwe:
                self.state = connection.state_string()
                log.debug("Handshake state: %s" % self.state)
                log.debug("doHandshake: WantWriteError on first handshake attempt.")
                connection = handleWantWrite(connection)
            else:
                self.state = connection.state_string()

            if self.state == 'SSL negotiation finished successfully':
                ## jump to handshakeSuccessful and get certchain
                return connection
            else:
                sent = connection.send("o\r\n")
                self.state = connection.state_string()
                log.debug("Handshake state: %s" % self.state)
                log.debug("Transmitted %d bytes" % sent)

                _read_buffer = connection.pending()
                log.debug("Max bytes in receive buffer: %d" % _read_buffer)

                try:
                    received = connection.recv(int(_read_buffer))
                except SSL.WantReadError, wre:
                    if connection.want_read():
                        self.state = connection.state_string()
                        connection = handleWantRead(connection)
                    else:
                        ## if we still have an SSL_ERROR_WANT_READ, then try to
                        ## renegotiate
                        self.state = connection.state_string()
                        connection = connectionRenegotiate(connection,
                                                           connection.getpeername(),
                                                           wre.message)
                except SSL.WantWriteError, wwe:
                    self.state = connection.state_string()
                    log.debug("Handshake state: %s" % self.state)
                    if connection.want_write():
                        connection = handleWantWrite(connection)
                    else:
                        raise ConnectionTimeout("Connection to %s:%d timed out."
                                                % (peername, peerport))
                else:
                    log.msg("Received: %s" % received)
                    self.state = connection.state_string()
                    log.debug("Handshake state: %s" % self.state)

            return connection

        def handshakeSucceeded(connection):
            """Get the details from the server certificate, cert chain, and
            server ciphersuite list, and put them in our report.

            WARNING: do *not* do this:
            >>> server_cert.get_pubkey()
                <OpenSSL.crypto.PKey at 0x4985d28>
            >>> pk = server_cert.get_pubkey()
            >>> pk.check()
                Segmentation fault

            :param connection: A :class:`OpenSSL.SSL.Connection <Connection>`.
            :returns: :param:`handshakeSucceeded.connection <connection>`.
            """
            host, port = connection.getpeername()
            log.msg("Handshake with %s:%d successful!" % (host, port))

            server_cert = self.getPeerCert(connection)
            server_cert_chain = self.getPeerCert(connection, get_chain=True)

            renegotiations = connection.total_renegotiations()
            cipher_list    = connection.get_cipher_list()
            session_key    = connection.master_key()
            rawcert        = connection.get_peer_certificate()
            ## xxx TODO this hash needs to be formatted as SHA1, not long
            cert_subj_hash = rawcert.subject_name_hash()
            cert_serial    = rawcert.get_serial_number()
            cert_sig_algo  = rawcert.get_signature_algorithm()
            cert_subject   = self.getX509Name(rawcert.get_subject(),
                                              get_components=True)
            cert_issuer    = self.getX509Name(rawcert.get_issuer(),
                                              get_components=True)
            cert_pubkey    = self.getPublicKey(rawcert.get_pubkey())

            self.report['host'] = host
            self.report['port'] = port
            self.report['state'] = self.state
            self.report['renegotiations'] = renegotiations
            self.report['server_cert'] = server_cert
            self.report['server_cert_chain'] = \
                ''.join([cert for cert in server_cert_chain])
            self.report['server_ciphersuite'] = cipher_list
            self.report['cert_subject'] = cert_subject
            self.report['cert_subj_hash'] = cert_subj_hash
            self.report['cert_issuer'] = cert_issuer
            self.report['cert_public_key'] = cert_pubkey
            self.report['cert_serial_no'] = cert_serial
            self.report['cert_sig_algo'] = cert_sig_algo
            ## The session's master key is only valid for that session, and
            ## will allow us to decrypt any packet captures (if they were
            ## collected). Because we are not requesting URLs, only host:port
            ## (which would be visible in pcaps anyway, since the FQDN is
            ## never encrypted) I do not see a way for this to log any user or
            ## identifying information. Correct me if I'm wrong.
            self.report['session_key'] = session_key

            log.msg("Server certificate:\n\n%s" % server_cert)
            log.msg("Server certificate chain:\n\n%s"
                    % ''.join([cert for cert in server_cert_chain]))
            log.msg("Negotiated ciphersuite:\n%s"
                    % '\n\t'.join([cipher for cipher in cipher_list]))
            log.msg("Certificate subject: %s" % cert_subject)
            log.msg("Certificate subject hash: %d" % cert_subj_hash)
            log.msg("Certificate issuer: %s" % cert_issuer)
            log.msg("Certificate public key:\n\n%s" % cert_pubkey)
            log.msg("Certificate signature algorithm: %s" % cert_sig_algo)
            log.msg("Certificate serial number: %s" % cert_serial)
            log.msg("Total renegotiations: %d" % renegotiations)

            return connection

        def handshakeFailed(connection, host):
            """Handle a failed handshake attempt and report the failure reason.

            :type connection: :class:`twisted.python.failure.Failure <Failure>`
                or :exc:Exception
            :param connection: The failed connection.
            :param tuple host: A tuple of the remote host's IP address as a
                string, and an integer specifying the remote host port, i.e.
                ('1.1.1.1',443)
            :returns: None
            """
            addr, port = host
            log.msg("Handshake with %s:%d failed!" % host)

            self.report['host'] = host
            self.report['port'] = port

            if isinstance(connection, Exception) \
                    or isinstance(connection, ConnectionTimeout):
                log.msg("Handshake failed with reason: %s" % connection.message)
                self.report['state'] = connection.message
            elif isinstance(connection, failure.Failure):
                log.msg("Handshake failed with reason: Socket %s"
                        % connection.getErrorMessage())
                self.report['state'] = connection.getErrorMessage()
                ctmo = connection.trap(ConnectionTimeout)
                if ctmo == ConnectionTimeout:
                    connection.cleanFailure()
            else:
                log.msg("Handshake failed with reason: %s" % str(connection))
                if not 'state' in self.report.keys():
                    self.report['state'] = str(connection)

            return None

        def deferMakeConnection(host):
            return threads.deferToThread(makeConnection, self.input)

        if self.host and not self.input:
            self.input = self.splitInput(self.host)
        log.msg("Beginning handshake test for %s:%s" % self.input)

        connection = deferMakeConnection(self.input)
        connection.addCallbacks(connectionSucceeded, connectionFailed,
                                callbackArgs=[self.input, self.timeout],
                                errbackArgs=[self.input])

        handshake = defer.Deferred()
        handshake.addCallback(doHandshake)
        handshake.addCallbacks(handshakeSucceeded, handshakeFailed,
                               errbackArgs=[self.input])

        connection.chainDeferred(handshake)
        connection.addCallbacks(connectionShutdown, defer.passthru,
                                callbackArgs=[self.input])
        connection.addBoth(log.exception)

        return connection



from twisted.internet import defer, reactor
from twisted.python import usage
from twisted.web.client import Agent, readBody
from ooni.templates.process import ProcessTest
from ooni.utils import log
from ooni.errors import handleAllFailures, failureToString

import distutils.spawn
import re


class UsageOptions(usage.Options):
    optParameters = [
        ['url', 'u', None, 'Specify a single URL on the OpenVPN subnet to test.'],
        ['openvpn-config', 'c', None, 'Specify an OpenVPN configuration file.'],
    ]


class OpenVPNTest(ProcessTest):

    """
    This class tests OpenVPN connections.

    test_openvpn_circumvent
      Starts an OpenVPN client on Linux and determines
      if it connects successfully to an OpenVPN server.
      Then, it make a HTTP request for http://google.com
      and records the response body or failure string.

    """

    name = "OpenVPN Client Test"
    description = "Connects to an OpenVPN server and does a HTTP GET for the" \
                  "specified URL."
    author = "srvetus "
    version = "0.0.2"
    timeout = 20
    usageOptions = UsageOptions
    requiredOptions = ['url', 'openvpn-config']
    requiresRoot = True

    def setUp(self):
        self.bootstrapped = defer.Deferred()
        self.command = [distutils.spawn.find_executable("openvpn")]
        self.exited = False
        self.url = self.localOptions.get('url')

        if self.localOptions.get('openvpn-config'):
            openvpn_config = self.localOptions.get('openvpn-config')
            self.command.extend(['--config', openvpn_config])

    def stop(self, reason=None):
        """Stop the running OpenVPN process and close the connection"""
        if not self.exited:
            self.processDirector.close()

            # OpenVPN needs to be sent SIGTERM to end cleanly
            self.processDirector.transport.signalProcess('TERM')
            self.exited = True

    def inConnectionLost(self):
            """Monkeypatch inConnectionLost to log failure if the process ends
            unexpectedly before OpenVPN bootstraps.
            """
            log.debug("inConnectionLost")

            if not self.bootstrapped.called:
                self.bootstrapped.errback(Exception("openvpn_exited_unexpectedly"))

    def processExited(self, reason):
            """Monkeypatch processExited to log failure if the process ends
            unexpectedly before OpenVPN bootstraps.
            """
            log.debug("Exited %s" % handleAllFailures(reason))

            # Process exited before OpenVPN bootstrapped. Add failure to report
            # if it is not added already
            if not self.bootstrapped.called:
                self.bootstrapped.errback(Exception("openvpn_exited_unexpectedly"))


    def handleRead(self, stdout=None, stderr=None):
        """handleRead is called with each chunk of data from stdout and stderr

        stdout only contains the latest data chunk, self.processDirector.stdout
        contains the combined stdout data.
        """

        # Read OpenVPN output until bootstrapping succeeds or fails
        if not self.bootstrapped.called:

            # TODO: Determine other OpenVPN messages which indicate connection failure
            if re.search(r'connect to .* failed', self.processDirector.stdout):
                log.debug("OpenVPN connection failed")

                # Bootstrapping failed
                self.bootstrapped.errback(Exception("openvpn_connection_failed"))

            # Check if OpenVPN has bootstrapped and connected successfully
            elif "Initialization Sequence Completed" in self.processDirector.stdout:
                log.debug("OpenVPN connection successful")
                self.processDirector.cancelTimer()
                self.bootstrapped.callback("bootstrapped")

    def test_openvpn_circumvent(self):

        def addResultToReport(result):
            log.debug("request_successful")
            self.report['body'] = result
            self.report['success'] = True

        def addFailureToReport(failure):
            log.debug("Failed: %s" % failureToString(failure))
            self.report['failure'] = failureToString(failure)
            self.report['success'] = False

        def doRequest(noreason):
            """Make a HTTP request over initialized VPN connection"""
            agent = Agent(reactor)

            log.debug("Doing HTTP request to the OpenVPN subnet: %s" % self.url)
            request = agent.request("GET", self.url)
            request.addCallback(readBody)
            request.addCallback(addResultToReport)
            request.addErrback(addFailureToReport)
            return request

        log.debug("Spawning OpenVPN")
        self.d = self.run(self.command)

        # Monkeypatch inConnectionLost and processExited to log when OpenVPN exits early
        self.processDirector.inConnectionLost = self.inConnectionLost
        self.processDirector.processExited = self.processExited

        # Try to make a request when the OpenVPN connection successfully bootstraps
        self.bootstrapped.addCallback(doRequest)

        # Fire failure if OpenVPN connection fails
        self.bootstrapped.addErrback(addFailureToReport)

        # Close OpenVPN after each successful or failed test
        self.bootstrapped.addBoth(self.stop)
        return self.d



import os
import shutil
from twisted.trial import unittest

from ooni.settings import config


class ConfigTestCase(unittest.TestCase):
    def setUp(self):
        self.ooni_home_dir = os.path.abspath("ooni_home")
        self.config = config
        self.config.initialize_ooni_home("ooni_home")
        super(ConfigTestCase, self).setUp()

    def skipTest(self, reason):
        raise unittest.SkipTest(reason)

    def tearDown(self):
        shutil.rmtree("ooni_home")



# -*- encoding: utf-8 -*-

from twisted.internet import defer
from twisted.python import usage

from ooni.templates import scapyt

from ooni.utils import log
from ooni.utils.txscapy import MPTraceroute
from ooni.settings import config


class UsageOptions(usage.Options):
    optParameters = [
                    ['backend', 'b', None, 'Test backend to use.'],
                    ['timeout', 't', 5, 'The timeout for the traceroute test.'],
                    ['maxttl', 'm', 30,
                     'The maximum value of ttl to set on packets.'],
                    ['dstport', 'd', None,
                     'Specify a single destination port. May be repeated.'],
                    ['interval', 'i', None,
                     'Specify the inter-packet delay in seconds.'],
                    ['numPackets', 'n', None,
                     'Specify the number of packets to send per hop.'],
        ]


class Traceroute(scapyt.BaseScapyTest):
    name = "Traceroute"
    description = "Performs a UDP, TCP, ICMP traceroute with destination port number "\
                  "set to 0, 22, 23, 53, 80, 123, 443, 8080 and 65535."

    requiredTestHelpers = {'backend': 'traceroute'}
    requiresRoot = True
    requiresTor = False

    usageOptions = UsageOptions
    dst_ports = [0, 22, 23, 53, 80, 123, 443, 8080, 65535]
    version = "0.3"

    def setUp(self):
        self.report['test_tcp_traceroute'] = dict(
            [('hops_%d' % d, []) for d in self.dst_ports])
        self.report['test_udp_traceroute'] = dict(
            [('hops_%d' % d, []) for d in self.dst_ports])
        self.report['test_icmp_traceroute'] = {'hops': []}

    @defer.inlineCallbacks
    def run_traceroute(self, protocol):
        st = MPTraceroute()
        if self.localOptions['maxttl']:
            st.ttl_max = int(self.localOptions['maxttl'])
        if self.localOptions['dstport']:
            st.dst_ports = [int(self.localOptions['dstport'])]
        if self.localOptions['interval']:
            st.interval = float(self.localOptions['interval'])
        log.msg("Running %s traceroute towards %s" % (protocol,
                                                      self.localOptions['backend']))
        log.msg("This will take about %s seconds" % st.timeout)
        config.scapyFactory.registerProtocol(st)
        traceroute = getattr(st, protocol + 'Traceroute')
        yield traceroute(self.localOptions['backend'])
        st.stopListening()
        st.matchResponses()
        for packet in st.sent_packets:
            self.report['sent_packets'].append(packet)
        for packet in st.matched_packets.values():
            self.report['answered_packets'].extend(packet)

        for ttl in xrange(st.ttl_min, st.ttl_max):
            matchedPackets = filter(
                lambda x: x.ttl == ttl,
                st.matched_packets.keys())
            for packet in matchedPackets:
                for response in st.matched_packets[packet]:
                    self.addToReport(packet, response)

    def test_icmp_traceroute(self):
        return self.run_traceroute('ICMP')

    def test_tcp_traceroute(self):
        return self.run_traceroute('TCP')

    def test_udp_traceroute(self):
        return self.run_traceroute('UDP')

    def addToReport(self, packet, response):
        if packet.proto == 1:
            self.report['test_icmp_traceroute']['hops'].append(
                {'ttl': packet.ttl, 'rtt': response.time - packet.time,
                 'address': response.src})
        elif packet.proto == 6:
            self.report['test_tcp_traceroute'][
                'hops_%s' % packet.dport].append(
                {'ttl': packet.ttl, 'rtt': response.time - packet.time,
                 'address': response.src, 'sport': response.sport})
        else:
            self.report['test_udp_traceroute'][
                'hops_%s' % packet.dport].append(
                {'ttl': packet.ttl, 'rtt': response.time - packet.time,
                 'address': response.src, 'sport': response.sport})



# -*- encoding: utf-8 -*-
import os
import random
import tempfile
import shutil

from twisted.python import usage
from twisted.internet import reactor, error

import txtorcon

from ooni.utils import log, onion
from ooni import nettest


class TorIsNotInstalled(Exception):
    pass


class UsageOptions(usage.Options):
    optParameters = [
        ['timeout', 't', 120,
         'Specify the timeout after which to consider '
         'the Tor bootstrapping process to have failed.'], ]


class BridgeReachability(nettest.NetTestCase):
    name = "Bridge Reachability"
    description = "A test for checking if bridges are reachable " \
                  "from a given location."
    author = "Arturo Filastò"
    version = "0.1.2"

    usageOptions = UsageOptions

    inputFile = ['file', 'f', None,
                 'File containing bridges to test reachability for. '
                 'They should be one per line IP:ORPort or '
                 'TransportType IP:ORPort (ex. obfs2 127.0.0.1:443).']

    requiredOptions = ['file']

    def requirements(self):
        if not onion.find_tor_binary():
            raise TorIsNotInstalled(
                "For instructions on installing Tor see: "
                "https://www.torproject.org/download/download")

    def setUp(self):
        self.tor_progress = 0
        self.timeout = int(self.localOptions['timeout'])

        fd, self.tor_logfile = tempfile.mkstemp()
        os.close(fd)
        fd, self.obfsproxy_logfile = tempfile.mkstemp()
        os.close(fd)
        self.tor_datadir = tempfile.mkdtemp()

        self.report['error'] = None
        self.report['success'] = None
        self.report['timeout'] = self.timeout
        self.report['transport_name'] = 'vanilla'
        self.report['tor_version'] = str(onion.tor_details['version'])
        self.report['tor_progress'] = 0
        self.report['tor_progress_tag'] = None
        self.report['tor_progress_summary'] = None
        self.report['tor_log'] = None
        self.report['obfsproxy_version'] = str(onion.obfsproxy_details['version'])
        self.report['obfsproxy_log'] = None
        self.report['bridge_address'] = None

        self.bridge = self.input
        if self.input.startswith('Bridge'):
            self.bridge = self.input.replace('Bridge ', '')

    def postProcessor(self, measurements):
        if 'successes' not in self.summary:
            self.summary['successes'] = []
        if 'failures' not in self.summary:
            self.summary['failures'] = []

        details = {
            'address': self.report['bridge_address'],
            'transport_name': self.report['transport_name'],
            'tor_progress': self.report['tor_progress']
        }
        if self.report['success']:
            self.summary['successes'].append(details)
        else:
            self.summary['failures'].append(details)
        return self.report

    def displaySummary(self, summary):
        successful_count = {}
        failure_count = {}

        def count(results, counter):
            for result in results:
                if result['transport_name'] not in counter:
                    counter[result['transport_name']] = 0
                counter[result['transport_name']] += 1
        count(summary['successes'], successful_count)
        count(summary['failures'], failure_count)

        working_bridges = ', '.join(
            ["%s %s" % (x['transport_name'], x['address'])
             for x in summary['successes']])
        failing_bridges = ', '.join(
            ["%s %s (at %s%%)"
             % (x['transport_name'], x['address'], x['tor_progress'])
             for x in summary['failures']])

        log.msg("Total successes: %d" % len(summary['successes']))
        log.msg("Total failures: %d" % len(summary['failures']))

        for transport, count in successful_count.items():
            log.msg("%s successes: %d" % (transport.title(), count))
        for transport, count in failure_count.items():
            log.msg("%s failures: %d" % (transport.title(), count))

        log.msg("Working bridges: %s" % working_bridges)
        log.msg("Failing bridges: %s" % failing_bridges)

    def test_full_tor_connection(self):
        config = txtorcon.TorConfig()
        config.ControlPort = random.randint(2**14, 2**16)
        config.SocksPort = random.randint(2**14, 2**16)
        config.DataDirectory = self.tor_datadir
        log.msg(
            "Connecting to %s with tor %s" %
            (self.bridge, onion.tor_details['version']))

        transport_name = onion.transport_name(self.bridge)
        if transport_name == None:
            self.report['bridge_address'] = self.bridge.split(' ')[0]
        else:
            self.report['bridge_address'] = self.bridge.split(' ')[1]
            self.report['transport_name'] = transport_name

            try:
                config.ClientTransportPlugin = \
                        onion.bridge_line(transport_name, self.obfsproxy_logfile)
            except onion.UnrecognizedTransport:
                log.err("Unable to test bridge because we don't recognize "
                        "the %s transport" % transport_name)
                self.report['error'] = "unrecognized-transport"
                return
            except onion.UninstalledTransport:
                bin_name = onion.transport_bin_name.get(transport_name)
                log.err("Unable to test bridge because %s is not installed" %
                        bin_name)
                self.report['error'] = "missing-%s" % bin_name
                return
            except onion.OutdatedObfsproxy:
                log.err("The obfsproxy version you are using " \
                        "appears to be outdated.")
                self.report['error'] = 'old-obfsproxy'
                return
            except onion.OutdatedTor:
                log.err("Unsupported Tor version.")
                self.report['error'] = 'unsupported-tor-version'
                return

            log.debug("Using ClientTransportPlugin '%s'" % \
                      config.ClientTransportPlugin)

        config.Bridge = self.bridge
        config.UseBridges = 1
        config.log = ['notice stdout', 'notice file %s' % self.tor_logfile]
        config.save()

        def updates(prog, tag, summary):
            log.msg("%s: %s%%" % (self.bridge, prog))
            self.report['tor_progress'] = int(prog)
            self.report['tor_progress_tag'] = tag
            self.report['tor_progress_summary'] = summary

        d = txtorcon.launch_tor(config, reactor, timeout=self.timeout,
                                progress_updates=updates)

        @d.addCallback
        def setup_complete(proto):
            try:
                proto.transport.signalProcess('TERM')
            except error.ProcessExitedAlready:
                proto.transport.loseConnection()
            log.msg("Successfully connected to %s" % self.bridge)
            self.report['success'] = True

        @d.addErrback
        def setup_failed(failure):
            log.msg("Failed to connect to %s" % self.bridge)
            self.report['success'] = False
            self.report['error'] = 'timeout-reached'
            return

        @d.addCallback
        def write_log(_):
            with open(self.tor_logfile) as f:
                self.report['tor_log'] = f.read()
            os.remove(self.tor_logfile)
            with open(self.obfsproxy_logfile) as f:
                self.report['obfsproxy_log'] = f.read()
            os.remove(self.obfsproxy_logfile)
            try:
                with open(os.path.join(self.tor_datadir,
                        'pt_state', 'obfs4proxy.log')) as f:
                    self.report['obfsproxy_log'] = f.read()
            except:
                pass
            finally:
                shutil.rmtree(self.tor_datadir)

        return d



import socket
from ooni.settings import config

config.initialize_ooni_home('ooni_home')
config.read_config_file()
config.logging = False
config.advanced.debug = False


def is_internet_connected():
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        s.connect(('torproject.org', 80))
        s.shutdown(2)
        return True
    except Exception:
        return False



import os
import yaml
import getpass
from ConfigParser import SafeConfigParser

from twisted.internet import defer, reactor
from twisted.internet.endpoints import TCP4ClientEndpoint

from os.path import abspath, expanduser

from ooni.utils.net import ConnectAndCloseProtocol, connectProtocol
from ooni import geoip
from ooni.utils import Storage, log, get_ooni_root
from ooni import errors


class OConfig(object):
    _custom_home = None

    def __init__(self):
        self.current_user = getpass.getuser()
        self.global_options = {}
        self.reports = Storage()
        self.scapyFactory = None
        self.tor_state = None
        # This is used to store the probes IP address obtained via Tor
        self.probe_ip = geoip.ProbeIP()
        self.logging = True
        self.basic = Storage()
        self.advanced = Storage()
        self.tor = Storage()
        self.privacy = Storage()
        self.set_paths()

    def embedded_settings(self, category, option):
        embedded_settings = os.path.join(get_ooni_root(), 'settings.ini')
        if os.path.isfile(embedded_settings):
            settings = SafeConfigParser()
            with open(embedded_settings) as fp:
                settings.readfp(fp)
            return settings.get(category, option)
        return None

    @property
    def var_lib_path(self):
        var_lib_path = self.embedded_settings("directories", "var_lib")
        if var_lib_path:
            return os.path.abspath(var_lib_path)
        return "/var/lib/ooni"

    @property
    def usr_share_path(self):
        usr_share_path = self.embedded_settings("directories", "usr_share")
        if usr_share_path:
            return os.path.abspath(usr_share_path)
        return "/usr/share/ooni"

    @property
    def data_directory_candidates(self):
        dirs = [
            self.ooni_home,
            self.var_lib_path,
            self.usr_share_path,
            os.path.join(get_ooni_root(), '..', 'data'),
            '/usr/share/'
        ]
        if os.getenv("OONI_DATA_DIR"):
            dirs.insert(0, os.getenv("OONI_DATA_DIR"))
        if self.global_options.get('datadir'):
            dirs.insert(0, abspath(expanduser(self.global_options['datadir'])))
        return dirs

    @property
    def data_directory(self):
        for target_dir in self.data_directory_candidates:
            if os.path.isdir(target_dir):
                return target_dir
        return self.var_lib_path

    @property
    def ooni_home(self):
        home = expanduser('~'+self.current_user)
        if os.getenv("HOME"):
            home = os.getenv("HOME")
        if self._custom_home:
            return self._custom_home
        else:
            return os.path.join(home, '.ooni')

    def get_data_file_path(self, file_name):
        for target_dir in self.data_directory_candidates:
            file_path = os.path.join(target_dir, file_name)
            if os.path.isfile(file_path):
                return file_path

    def set_paths(self):
        self.nettest_directory = os.path.join(get_ooni_root(), 'nettests')

        if self.advanced.inputs_dir:
            self.inputs_directory = self.advanced.inputs_dir
        else:
            self.inputs_directory = os.path.join(self.ooni_home, 'inputs')

        if self.advanced.decks_dir:
            self.decks_directory = self.advanced.decks_dir
        else:
            self.decks_directory = os.path.join(self.ooni_home, 'decks')

        self.reports_directory = os.path.join(self.ooni_home, 'reports')
        self.resources_directory = os.path.join(self.data_directory,
                                                "resources")
        if self.advanced.report_log_file:
            self.report_log_file = self.advanced.report_log_file
        else:
            self.report_log_file = os.path.join(self.ooni_home,
                                                'reporting.yml')

        if self.global_options.get('configfile'):
            config_file = self.global_options['configfile']
            self.config_file = expanduser(config_file)
        else:
            self.config_file = os.path.join(self.ooni_home, 'ooniprobe.conf')

        if 'logfile' in self.basic:
            self.basic.logfile = expanduser(self.basic.logfile.replace(
                '~', '~'+self.current_user))

    def initialize_ooni_home(self, custom_home=None):
        if custom_home:
            self._custom_home = custom_home
            self.set_paths()

        if not os.path.isdir(self.ooni_home):
            print "Ooni home directory does not exist."
            print "Creating it in '%s'." % self.ooni_home
            os.mkdir(self.ooni_home)
            os.mkdir(self.inputs_directory)
            os.mkdir(self.decks_directory)

    def _create_config_file(self):
        target_config_file = self.config_file
        print "Creating it for you in '%s'." % target_config_file
        sample_config_file = self.get_data_file_path('ooniprobe.conf.sample')

        with open(sample_config_file) as f:
            with open(target_config_file, 'w+') as w:
                for line in f:
                    if line.startswith('    logfile: '):
                        w.write('    logfile: %s\n' % (
                            os.path.join(self.ooni_home, 'ooniprobe.log'))
                        )
                    else:
                        w.write(line)

    def read_config_file(self, check_incoherences=False):
        if not os.path.isfile(self.config_file):
            print "Configuration file does not exist."
            self._create_config_file()
            self.read_config_file()

        with open(self.config_file) as f:
            config_file_contents = '\n'.join(f.readlines())
            configuration = yaml.safe_load(config_file_contents)

        for setting in configuration.keys():
            if setting in dir(self) and configuration[setting] is not None:
                for k, v in configuration[setting].items():
                    getattr(self, setting)[k] = v

        self.set_paths()
        if check_incoherences:
            self.check_incoherences(configuration)

    def check_incoherences(self, configuration):
        incoherent = []

        if configuration['advanced']['interface'] != 'auto':
            from scapy.all import get_if_list
            if configuration['advanced']['interface'] not in get_if_list():
                incoherent.append('advanced:interface')

        self.log_incoherences(incoherent)

    def log_incoherences(self, incoherences):
        if len(incoherences) > 0:
            if len(incoherences) > 1:
                incoherent_pretty = ", ".join(incoherences[:-1]) + ' and ' + incoherences[-1]
            else:
                incoherent_pretty = incoherences[0]
            log.err("You must set properly %s in %s." % (incoherent_pretty, self.config_file))
            raise errors.ConfigFileIncoherent

    @defer.inlineCallbacks
    def check_tor(self):
        """
        Called only when we must start tor by director.start
        """
        incoherent = []
        if not self.advanced.start_tor:
            if self.tor.socks_port is None:
                incoherent.append('tor:socks_port')
            else:
                socks_port_ep = TCP4ClientEndpoint(reactor,
                                                   "localhost",
                                                   self.tor.socks_port)
                try:
                    yield connectProtocol(socks_port_ep, ConnectAndCloseProtocol())
                except Exception:
                    incoherent.append('tor:socks_port')

            if self.tor.control_port is not None:
                control_port_ep = TCP4ClientEndpoint(reactor,
                                                     "localhost",
                                                     self.tor.control_port)
                try:
                    yield connectProtocol(control_port_ep, ConnectAndCloseProtocol())
                except Exception:
                    incoherent.append('tor:control_port')

            self.log_incoherences(incoherent)

config = OConfig()
if not os.path.isfile(config.config_file) \
       and os.path.isfile('/etc/ooniprobe.conf'):
    config.global_options['configfile'] = '/etc/ooniprobe.conf'
    config.set_paths()



"""
how this works
--------------

This classifier uses the DOM structure of a website to determine how similar
the two sites are.
The procedure we use is the following:
   * First we parse all the DOM tree of the web page and we build a list of
     TAG parent child relationships (ex. <html><a><b></b></a><c></c></html> =>
     (html, a), (a, b), (html, c)).

   * We then use this information to build a matrix (M) where m[i][j] = P(of
     transitioning from tag[i] to tag[j]). If tag[i] does not exists P() = 0.
     Note: M is a square matrix that is number_of_tags wide.

   * We then calculate the eigenvectors (v_i) and eigenvalues (e) of M.

   * The corelation between page A and B is given via this formula:
     correlation = dot_product(e_A, e_B), where e_A and e_B are
     resepectively the eigenvalues for the probability matrix A and the
     probability matrix B.
"""

import numpy
import time

from ooni import log

# All HTML4 tags
# XXX add link to W3C page where these came from
alltags = ['A', 'ABBR', 'ACRONYM', 'ADDRESS', 'APPLET', 'AREA', 'B', 'BASE',
           'BASEFONT', 'BD', 'BIG', 'BLOCKQUOTE', 'BODY', 'BR', 'BUTTON', 'CAPTION',
           'CENTER', 'CITE', 'CODE', 'COL', 'COLGROUP', 'DD', 'DEL', 'DFN', 'DIR', 'DIV',
           'DL', 'DT', 'E M', 'FIELDSET', 'FONT', 'FORM', 'FRAME', 'FRAMESET', 'H1', 'H2',
           'H3', 'H4', 'H5', 'H6', 'HEAD', 'HR', 'HTML', 'I', 'IFRAME ', 'IMG',
           'INPUT', 'INS', 'ISINDEX', 'KBD', 'LABEL', 'LEGEND', 'LI', 'LINK', 'MAP',
           'MENU', 'META', 'NOFRAMES', 'NOSCRIPT', 'OBJECT', 'OL', 'OPTGROUP', 'OPTION',
           'P', 'PARAM', 'PRE', 'Q', 'S', 'SAMP', 'SCRIPT', 'SELECT', 'SMALL', 'SPAN',
           'STRIKE', 'STRONG', 'STYLE', 'SUB', 'SUP', 'TABLE', 'TBODY', 'TD',
           'TEXTAREA', 'TFOOT', 'TH', 'THEAD', 'TITLE', 'TR', 'TT', 'U', 'UL', 'VAR']

# Reduced subset of only the most common tags
commontags = ['A', 'B', 'BLOCKQUOTE', 'BODY', 'BR', 'BUTTON', 'CAPTION',
           'CENTER', 'CITE', 'CODE', 'COL', 'DD', 'DIV',
           'DL', 'DT', 'EM', 'FIELDSET', 'FONT', 'FORM', 'FRAME', 'FRAMESET', 'H1', 'H2',
           'H3', 'H4', 'H5', 'H6', 'HEAD', 'HR', 'HTML', 'IFRAME ', 'IMG',
           'INPUT', 'INS', 'LABEL', 'LEGEND', 'LI', 'LINK', 'MAP',
           'MENU', 'META', 'NOFRAMES', 'NOSCRIPT', 'OBJECT', 'OL', 'OPTION',
           'P', 'PRE', 'SCRIPT', 'SELECT', 'SMALL', 'SPAN',
           'STRIKE', 'STRONG', 'STYLE', 'SUB', 'SUP', 'TABLE', 'TBODY', 'TD',
           'TEXTAREA', 'TFOOT', 'TH', 'THEAD', 'TITLE', 'TR', 'TT', 'U', 'UL']

# The tags we are intested in using for our analysis
thetags = ['A', 'DIV', 'FRAME', 'H1', 'H2',
           'H3', 'H4', 'IFRAME ', 'INPUT',
           'LABEL','LI', 'P', 'SCRIPT', 'SPAN',
           'STYLE', 'TR']

def compute_probability_matrix(dataset):
    """
    Compute the probability matrix based on the input dataset.

    :dataset: an array of pairs representing the parent child relationships.
    """
    matrix = numpy.zeros((len(thetags) + 1, len(thetags) + 1))

    for data in dataset:
        x = data[0].upper()
        y = data[1].upper()
        try:
            x = thetags.index(x)
        except:
            x = len(thetags)

        try:
            y = thetags.index(y)
        except:
            y = len(thetags)

        matrix[x,y] += 1

    for x in xrange(len(thetags) + 1):
        possibilities = 0
        for y in matrix[x]:
            possibilities += y

        for i in xrange(len(matrix[x])):
            if possibilities != 0:
                matrix[x][i] = matrix[x][i]/possibilities

    return matrix

def compute_eigenvalues(matrix):
    """
    Returns the eigenvalues of the supplied square matrix.

    :matrix: must be a square matrix and diagonalizable.
    """
    return numpy.linalg.eigvals(matrix)

def readDOM(content=None, filename=None, debug=False):
    """
    Parses the DOM of the HTML page and returns an array of parent, child
    pairs.

    :content: the content of the HTML page to be read.

    :filename: the filename to be read from for getting the content of the
               page.
    """
    try:
        from bs4 import BeautifulSoup
    except ImportError:
        log.err("BeautifulSoup is not installed. This test canno run")
        raise Exception

    if filename:
        f = open(filename)
        content = ''.join(f.readlines())
        f.close()

    if debug:
        start = time.time()
        print "Running BeautifulSoup on content"
    dom = BeautifulSoup(content)
    if debug:
        print "done in %s" % (time.time() - start)

    if debug:
        start = time.time()
        print "Creating couples matrix"
    couples = []
    for x in dom.findAll():
        couples.append((str(x.parent.name), str(x.name)))
    if debug:
        print "done in %s" % (time.time() - start)

    return couples

def compute_eigenvalues_from_DOM(*arg,**kw):
    dom = readDOM(*arg, **kw)
    probability_matrix = compute_probability_matrix(dom)
    eigenvalues = compute_eigenvalues(probability_matrix)
    return eigenvalues

def compute_correlation(matrix_a, matrix_b):
    correlation = numpy.vdot(matrix_a, matrix_b)
    correlation /= numpy.linalg.norm(matrix_a)*numpy.linalg.norm(matrix_b)
    correlation = (correlation + 1)/2
    return correlation

def benchmark():
    """
    Running some very basic benchmarks on this input data:

    Data files:
    683 filea.txt
    678 fileb.txt

    diff file* | wc -l
    283

    We get such results:

    Read file B
    Running BeautifulSoup on content
    done in 0.768223047256
    Creating couples matrix
    done in 0.023903131485
    --------
    total done in 0.796372890472
    Read file A
    Running BeautifulSoup on content
    done in 0.752885818481
    Creating couples matrix
    done in 0.0163578987122
    --------
    total done in 0.770951986313
    Computing prob matrix
    done in 0.0475239753723
    Computing eigenvalues
    done in 0.00161099433899
    Computing prob matrix B
    done in 0.0408289432526
    Computing eigen B
    done in 0.000268936157227
    Computing correlation
    done in 0.00016713142395
    Corelation: 0.999999079331

    What this means is that the bottleneck is not in the maths, but is rather
    in the computation of the DOM tree matrix.

    XXX We should focus on optimizing the parsing of the HTML (this depends on
    beautiful soup). Perhaps we can find and alternative to it that is
    sufficient for us.
    """
    start = time.time()
    print "Read file B"
    site_a = readDOM(filename='filea.txt', debug=True)
    print "--------"
    print "total done in %s" % (time.time() - start)

    start = time.time()
    print "Read file A"
    site_b = readDOM(filename='fileb.txt', debug=True)
    print "--------"
    print "total done in %s" % (time.time() - start)

    a = {}
    b = {}

    start = time.time()
    print "Computing prob matrix"
    a['matrix'] = compute_probability_matrix(site_a)
    print "done in %s" % (time.time() - start)
    start = time.time()

    print "Computing eigenvalues"
    a['eigen'] = compute_eigenvalues(a['matrix'])
    print "done in %s" % (time.time() - start)
    start = time.time()

    start = time.time()
    print "Computing prob matrix B"
    b['matrix'] = compute_probability_matrix(site_b)
    print "done in %s" % (time.time() - start)

    start = time.time()
    print "Computing eigen B"
    b['eigen'] = compute_eigenvalues(b['matrix'])
    print "done in %s" % (time.time() - start)

    start = time.time()
    print "Computing correlation"
    correlation = compute_correlation(a['eigen'], b['eigen'])
    print "done in %s" % (time.time() - start)

    print "Corelation: %s" % correlation

#benchmark()



import os
import sys
import copy
import errno

import yaml

from twisted.internet import defer
from twisted.python import usage

from ooni import errors
from ooni.geoip import ProbeIP
from ooni.settings import config

from ooni.deckgen import __version__
from ooni.deckgen.processors import citizenlab_test_lists
from ooni.deckgen.processors import namebench_dns_servers
from ooni.resources.update import download_resources

class Options(usage.Options):
    synopsis = """%s [options]
    """ % sys.argv[0]

    optParameters = [
        ["country-code", "c", None,
         "Specify the two letter country code for which we should "
         "generate the deck."],
        ["collector", None, None, "Specify a custom collector to use when "
                                  "submitting reports"],
        ["bouncer", None, None, "Specify a custom bouncer to use"],
        ["output", "o", None,
         "Specify the directory where to write output."]
    ]

    def opt_version(self):
        print("oonideckgen version: %s" % __version__)
        sys.exit(0)


class Deck(object):
    _base_entry = {
        "options": {
            "test_file": None,
            "subargs": [],
            "annotations": None,

            "collector": None,
            "bouncer": None,

            "reportfile": None,

            "no-collector": 0,
            "no-geoip": 0,
            "no-yamloo": 0,
            "verbose": 0
        }
    }

    def __init__(self, collector=None, bouncer=None):
        self.deck_entries = []
        self.collector = collector
        self.bouncer = bouncer

    def add_test(self, test_file, subargs=[]):
        deck_entry = copy.deepcopy(self._base_entry)
        deck_entry['options']['collector'] = self.collector
        deck_entry['options']['bouncer'] = self.bouncer
        deck_entry['options']['test_file'] = test_file
        deck_entry['options']['subargs'] = subargs
        self.deck_entries.append(deck_entry)

    def pprint(self):
        print yaml.safe_dump(self.deck_entries)

    def write_to_file(self, filename):
        with open(filename, "w+") as f:
            f.write(yaml.safe_dump(self.deck_entries))


def generate_deck(options):
    url_list_country = None
    try:
        url_list_country = citizenlab_test_lists.generate_country_input(
            options['country-code'],
            options['output']
        )
    except Exception:
        print "Could not generate country specific url list"
        print "We will just use the global one."

    url_list_global = citizenlab_test_lists.generate_global_input(
        options['output']
    )

    deck = Deck(collector=options['collector'], bouncer=options['bouncer'])
    deck.add_test('manipulation/http_invalid_request_line')
    deck.add_test('manipulation/http_header_field_manipulation')

    if url_list_country is not None:
        deck.add_test('blocking/web_connectivity', ['-f', url_list_country])
    deck.add_test('blocking/web_connectivity', ['-f', url_list_global])

    if config.advanced.debug:
        deck.pprint()
    deck_filename = os.path.join(options['output'],
                                 "default-user.deck")
    deck.write_to_file(deck_filename)
    print "Deck written to %s" % deck_filename
    print "Run ooniprobe like so:"
    print "ooniprobe -i %s" % deck_filename


@defer.inlineCallbacks
def get_user_country_code():
    config.privacy.includecountry = True
    probe_ip = ProbeIP()
    yield probe_ip.lookup()
    defer.returnValue(probe_ip.geodata['countrycode'])

def resources_up_to_date():
    if config.get_data_file_path("GeoIP/GeoIP.dat") is None:
        return False

    if config.get_data_file_path("resources/"
                                 "namebench-dns-servers.csv") is None:
        return False

    if config.get_data_file_path("resources/"
                                 "citizenlab-test-lists/"
                                 "global.csv") is None:
        return False

    return True

@defer.inlineCallbacks
def run():
    options = Options()
    try:
        options.parseOptions()
    except usage.UsageError as error_message:
        print "%s: %s" % (sys.argv[0], error_message)
        print options
        sys.exit(1)

    if not resources_up_to_date():
        print("Resources for running ooniprobe are not up to date.")
        print("Will update them now.")
        yield download_resources()

    if not options['output']:
        options['output'] = os.getcwd()

    if not options['country-code']:
        try:
            options['country-code'] = yield get_user_country_code()
        except errors.ProbeIPUnknown:
            print "Could not determine your IP address."
            print "Check your internet connection or specify a country code with -c."
            sys.exit(4)

    if len(options['country-code']) != 2:
        print "%s: --country-code must be 2 characters" % sys.argv[0]
        sys.exit(2)

    if not os.path.isdir(options['output']):
        print "%s: %s is not a directory" % (sys.argv[0],
                                             options['output'])
        sys.exit(3)

    options['country-code'] = options['country-code'].lower()

    output_dir = os.path.abspath(options['output'])
    output_dir = os.path.join(output_dir,
                              "deck-%s" % options['country-code'])
    options['output'] = output_dir

    try:
        os.makedirs(options['output'])
    except OSError as exception:
        if exception.errno != errno.EEXIST:
            raise

    generate_deck(options)



import os
import sys
import yaml

from twisted.internet import defer

import exceptions
from ooni import errors
from ooni.tests import is_internet_connected
from ooni.tests.bases import ConfigTestCase
from ooni.settings import config
from ooni.oonicli import runWithDirector, setupGlobalOptions
from ooni.oonicli import setupAnnotations, setupCollector
from ooni.oonicli import createDeck
from ooni.utils.net import hasRawSocketPermission


def verify_header(header):
    assert 'input_hashes' in header.keys()
    assert 'options' in header.keys()
    assert 'probe_asn' in header.keys()
    assert 'probe_cc' in header.keys()
    assert 'probe_ip' in header.keys()
    assert 'software_name' in header.keys()
    assert 'software_version' in header.keys()
    assert 'test_name' in header.keys()
    assert 'test_version' in header.keys()


def verify_entry(entry):
    assert 'input' in entry

config_includepcap = """
basic:
    logfile: ~/.ooni/ooniprobe.log
privacy:
    includeip: false
    includeasn: true
    includecountry: true
    includepcap: true
reports:
    pcap: null
    collector: null
advanced:
    debug: false
    interface: auto
    start_tor: false
    measurement_timeout: 60
    measurement_retries: 2
    measurement_concurrency: 10
    reporting_timeout: 80
    reporting_retries: 3
    reporting_concurrency: 15
    data_dir: %s
    oonid_api_port: 8042
tor:
    socks_port: 9050

""" % config.data_directory


class TestRunDirector(ConfigTestCase):
    timeout = 220

    def setUp(self):
        super(TestRunDirector, self).setUp()
        if not is_internet_connected():
            self.skipTest("You must be connected to the internet to run this test")

        config.tor.socks_port = 9050
        config.tor.control_port = None
        self.filenames = ['example-input.txt']
        with open('example-input.txt', 'w+') as f:
            f.write('http://torproject.org/\n')
            f.write('http://bridges.torproject.org/\n')
            f.write('http://blog.torproject.org/\n')

    def tearDown(self):
        super(TestRunDirector, self).tearDown()
        for filename in self.filenames:
            if os.path.exists(filename):
                os.remove(filename)
        self.filenames = []

    @defer.inlineCallbacks
    def run_helper(self, test_name, nettest_args, verify_function, ooni_args=()):
        output_file = os.path.abspath('test_report.yamloo')
        self.filenames.append(output_file)
        oldargv = sys.argv
        sys.argv = ['']
        sys.argv.extend(ooni_args)
        sys.argv.extend(['-n', '-o', output_file, test_name])
        sys.argv.extend(nettest_args)
        global_options = setupGlobalOptions(False, False, False)
        yield runWithDirector(global_options)
        with open(output_file) as f:
            entries = yaml.safe_load_all(f)
            header = entries.next()
            try:
                first_entry = entries.next()
            except StopIteration:
                raise Exception("Missing entry in report")
        verify_header(header)
        verify_entry(first_entry)
        verify_function(first_entry)
        sys.argv = oldargv

    @defer.inlineCallbacks
    def test_http_requests(self):
        def verify_function(entry):
            assert 'body_length_match' in entry
            assert 'body_proportion' in entry
            assert 'control_failure' in entry
            assert 'experiment_failure' in entry
            assert 'factor' in entry
            assert 'headers_diff' in entry
            assert 'headers_match' in entry

        yield self.run_helper('blocking/http_requests',
                              ['-u', 'http://torproject.org/'],
                              verify_function)

    @defer.inlineCallbacks
    def test_http_requests_with_file(self):
        def verify_function(entry):
            assert 'body_length_match' in entry
            assert 'body_proportion' in entry
            assert 'control_failure' in entry
            assert 'experiment_failure' in entry
            assert 'factor' in entry
            assert 'headers_diff' in entry
            assert 'headers_match' in entry

        yield self.run_helper('blocking/http_requests',
                              ['-f', 'example-input.txt'],
                              verify_function)

    @defer.inlineCallbacks
    def test_dnsconsistency(self):
        def verify_function(entry):
            assert 'queries' in entry
            assert 'control_resolver' in entry
            assert 'errors' in entry
            assert 'inconsistent' in entry
            assert 'failures' in entry
            assert 'successful' in entry
            assert len(entry['inconsistent']) == 0

        yield self.run_helper('blocking/dns_consistency',
                              ['-b', '8.8.8.8:53',
                               '-t', '8.8.8.8',
                               '-f', 'example-input.txt'],
                              verify_function)

    @defer.inlineCallbacks
    def test_http_header_field_manipulation(self):
        self.skipTest("This test requires a property configured backend")
        def verify_function(entry):
            assert 'agent' in entry
            assert 'requests' in entry
            assert 'socksproxy' in entry
            assert 'errors' in entry

        yield self.run_helper('manipulation/http_header_field_manipulation',
                              ['-b', 'http://4.15.35.157:80'],
                              verify_function)

    @defer.inlineCallbacks
    def test_sniffing_activated(self):
        if not hasRawSocketPermission():
            self.skipTest("You must run this test as root or have the "
                          "capabilities cap_net_admin,cap_net_raw+eip")
        self.skipTest("Not properly set packet capture?")
        filename = os.path.abspath('test_report.pcap')
        self.filenames.append(filename)
        conf_file = os.path.abspath('fake_config.conf')
        with open(conf_file, 'w') as cfg:
            cfg.writelines(config_includepcap)
        self.filenames.append(conf_file)

        def verify_function(_):
            assert os.path.exists(filename)
            self.assertGreater(os.stat(filename).st_size, 0)
        yield self.run_helper('blocking/http_requests',
                              ['-f', 'example-input.txt'],
                              verify_function, ooni_args=['-f', conf_file])
        config.scapyFactory.connectionLost('')


class TestOoniCli(ConfigTestCase):
    def test_create_deck_not_found(self):
        global_options = {
            'no-yamloo': True,
            'subargs': [],
            'annotations': {},
            'collector': None,
            'bouncer': None,
            'no-collector': False,
            'test_file': 'invalid/path',
            'testdeck': None
        }
        exc = None
        try:
            createDeck(global_options)
        except exceptions.SystemExit as exc:
            exc = exc
        self.assertIsNotNone(exc)
        self.assertEqual(exc.code, 3)

    def test_setup_annotations(self):
        global_options = {
            "annotations": "key1:value1,key2:value2"
        }
        annotations = setupAnnotations(global_options)
        self.assertEqual(annotations,
                         {'key1': 'value1', 'key2': 'value2'})

    def test_setup_collector(self):
        collector1 = 'https://example.com'
        collector2 = 'httpo://thirteenchars321.onion'
        global_options = {
            'collector': collector1
        }
        try:
            collector_client = setupCollector(global_options, collector2)
            self.assertEqual(collector_client.settings['address'], collector1)
            self.assertEqual(collector_client.settings['type'], 'https')
        except errors.CollectorUnsupported:
            # Older versions of twisted will raise this. We could be more
            # strict and do a check for older twisted versions in here.
            pass



from ooni import nettest
from ooni.utils import log
from twisted.internet import defer, protocol, reactor
from twisted.python import usage

import os


def which(program):
    def is_exe(fpath):
        return os.path.isfile(fpath) and os.access(fpath, os.X_OK)

    fpath, fname = os.path.split(program)
    if fpath:
        if is_exe(program):
            return program
    else:
        for path in os.environ["PATH"].split(os.pathsep):
            path = path.strip('"')
            exe_file = os.path.join(path, program)
            if is_exe(exe_file):
                return exe_file
    return None


class UsageOptions(usage.Options):
    optParameters = [
        ['interpreter', 'i', '', 'The interpreter to use'],
        ['script', 's', '', 'The script to run']
    ]


class ScriptProcessProtocol(protocol.ProcessProtocol):
    def __init__(self, test_case):
        self.test_case = test_case
        self.deferred = defer.Deferred()

    def connectionMade(self):
        log.debug("connectionMade")
        self.transport.closeStdin()
        self.test_case.report['lua_output'] = ""

    def outReceived(self, data):
        log.debug('outReceived: %s' % data)
        self.test_case.report['lua_output'] += data

    def errReceived(self, data):
        log.err('Script error: %s' % data)
        self.transport.signalProcess('KILL')

    def processEnded(self, status):
        rc = status.value.exitCode
        log.debug('processEnded: %s, %s' % \
                  (rc, self.test_case.report['lua_output']))
        if rc == 0:
            self.deferred.callback(self)
        else:
            self.deferred.errback(rc)


# TODO: Maybe the script requires a back-end.
class Script(nettest.NetTestCase):
    name = "Script test"
    version = "0.1"
    authors = "Dominic Hamon"

    usageOptions = UsageOptions
    requiredOptions = ['interpreter', 'script']
    requiresRoot = False
    requiresTor = False

    def test_run_script(self):
        """
        We run the script specified in the usage options and take whatever
        is printed to stdout as the results of the test.
        """
        processProtocol = ScriptProcessProtocol(self)

        interpreter = self.localOptions['interpreter']
        if not which(interpreter):
            log.err('Unable to find %s executable in PATH.' % interpreter)
            return

        reactor.spawnProcess(processProtocol,
                             interpreter,
                             args=[interpreter, self.localOptions['script']],
                             env={'HOME': os.environ['HOME']},
                             usePTY=True)

        if not reactor.running:
            reactor.run()
        return processProtocol.deferred



import sys

import os
import json
import yaml
import random
import textwrap
import urlparse

from twisted.python import usage
from twisted.internet import defer

from ooni import errors, __version__
from ooni.constants import CANONICAL_BOUNCER_ONION
from ooni.settings import config
from ooni.utils import log

class LifetimeExceeded(Exception): pass

class Options(usage.Options):
    synopsis = """%s [options] [path to test].py
    """ % (os.path.basename(sys.argv[0]),)

    longdesc = ("ooniprobe loads and executes a suite or a set of suites of"
                " network tests. These are loaded from modules, packages and"
                " files listed on the command line.")

    optFlags = [["help", "h"],
                ["no-collector", "n", "Disable writing to collector"],
                ["no-yamloo", "N", "Disable writing to YAML file"],
                ["no-geoip", "g", "Disable geoip lookup on start"],
                ["list", "s", "List the currently installed ooniprobe "
                              "nettests"],
                ["printdeck", "p", "Print the equivalent deck for the "
                                   "provided command"],
                ["verbose", "v", "Show more verbose information"]
                ]

    optParameters = [
        ["reportfile", "o", None, "Specify the report file name to write to."],
        ["testdeck", "i", None, "Specify as input a test deck: a yaml file "
                                "containing the tests to run and their "
                                "arguments."],
        ["collector", "c", None, "Specify the address of the collector for "
                                 "test results. In most cases a user will "
                                 "prefer to specify a bouncer over this."],
        ["bouncer", "b", None, "Specify the bouncer used to "
                               "obtain the address of the "
                               "collector and test helpers."],
        ["logfile", "l", None, "Write to this logs to this filename."],
        ["pcapfile", "O", None, "Write a PCAP of the ooniprobe session to "
                                "this filename."],
        ["configfile", "f", None, "Specify a path to the ooniprobe "
                                  "configuration file."],
        ["datadir", "d", None, "Specify a path to the ooniprobe data "
                               "directory."],
        ["annotations", "a", None, "Annotate the report with a key:value[, "
                                   "key:value] format."],
        ["preferred-backend", "P", None, "Set the preferred backend to use "
                                         "when submitting results and/or "
                                         "communicating with test helpers. "
                                         "Can be either onion, "
                                         "https or cloudfront"],
        ["queue", "Q", None, "AMQP Queue URL amqp://user:pass@host:port/vhost/queue"]
    ]

    compData = usage.Completions(
        extraActions=[usage.CompleteFiles(
            "*.py", descr="file | module | package | TestCase | testMethod",
            repeat=True)],)

    tracer = None

    def __init__(self):
        usage.Options.__init__(self)

    def getUsage(self, width=None):
        return super(Options, self).getUsage(width) + """
To get started you may want to run:

$ oonideckgen

This will tell you how to run ooniprobe :)
"""

    def opt_spew(self):
        """
        Print an insanely verbose log of everything that happens.
        Useful when debugging freezes or locks in complex code.
        """
        from twisted.python.util import spewer
        sys.settrace(spewer)

    def opt_version(self):
        """
        Display the ooniprobe version and exit.
        """
        print "ooniprobe version:", __version__
        sys.exit(0)

    def parseArgs(self, *args):
        if self['testdeck'] or self['list']:
            return
        try:
            self['test_file'] = args[0]
            self['subargs'] = args[1:]
        except:
            raise usage.UsageError("No test filename specified!")


def parseOptions():
    print "WARNING: running ooniprobe involves some risk that varies greatly"
    print "         from country to country. You should be aware of this when"
    print "         running the tool. Read more about this in the manpage or README."
    cmd_line_options = Options()
    if len(sys.argv) == 1:
        cmd_line_options.getUsage()
    try:
        cmd_line_options.parseOptions()
    except usage.UsageError as ue:
        print cmd_line_options.getUsage()
        raise SystemExit("%s: %s" % (sys.argv[0], ue))

    return dict(cmd_line_options)


def director_startup_handled_failures(failure):
    log.err("Could not start the director")
    failure.trap(errors.TorNotRunning,
                 errors.InvalidOONIBCollectorAddress,
                 errors.UnableToLoadDeckInput,
                 errors.CouldNotFindTestHelper,
                 errors.CouldNotFindTestCollector,
                 errors.ProbeIPUnknown,
                 errors.InvalidInputFile,
                 errors.ConfigFileIncoherent)

    if isinstance(failure.value, errors.TorNotRunning):
        log.err("Tor does not appear to be running")
        log.err("Reporting with a collector is not possible")
        log.msg(
            "Try with a different collector or disable collector reporting with -n")

    elif isinstance(failure.value, errors.InvalidOONIBCollectorAddress):
        log.err("Invalid format for oonib collector address.")
        log.msg(
            "Should be in the format http://<collector_address>:<port>")
        log.msg("for example: ooniprobe -c httpo://nkvphnp3p6agi5qq.onion")

    elif isinstance(failure.value, errors.UnableToLoadDeckInput):
        log.err("Unable to fetch the required inputs for the test deck.")
        log.msg(
            "Please file a ticket on our issue tracker: https://github.com/thetorproject/ooni-probe/issues")

    elif isinstance(failure.value, errors.CouldNotFindTestHelper):
        log.err("Unable to obtain the required test helpers.")
        log.msg(
            "Try with a different bouncer or check that Tor is running properly.")

    elif isinstance(failure.value, errors.CouldNotFindTestCollector):
        log.err("Could not find a valid collector.")
        log.msg(
            "Try with a different bouncer, specify a collector with -c or disable reporting to a collector with -n.")

    elif isinstance(failure.value, errors.ProbeIPUnknown):
        log.err("Failed to lookup probe IP address.")
        log.msg("Check your internet connection.")

    elif isinstance(failure.value, errors.InvalidInputFile):
        log.err("Invalid input file \"%s\"" % failure.value)

    elif isinstance(failure.value, errors.ConfigFileIncoherent):
        log.err("Incoherent config file")

    if config.advanced.debug:
        log.exception(failure)

def director_startup_other_failures(failure):
    log.err("An unhandled exception occurred while starting the director!")
    log.exception(failure)

def setupGlobalOptions(logging, start_tor, check_incoherences):
    global_options = parseOptions()

    config.global_options = global_options
    config.set_paths()
    config.initialize_ooni_home()
    try:
        config.read_config_file(check_incoherences=check_incoherences)
    except errors.ConfigFileIncoherent:
        sys.exit(6)

    if global_options['verbose']:
        config.advanced.debug = True

    if not start_tor:
        config.advanced.start_tor = False

    if logging:
        log.start(global_options['logfile'])

    if config.privacy.includepcap or global_options['pcapfile']:
        from ooni.utils.net import hasRawSocketPermission
        if hasRawSocketPermission():
            from ooni.utils.txscapy import ScapyFactory
            config.scapyFactory = ScapyFactory(config.advanced.interface)
        else:
            log.err("Insufficient Privileges to capture packets."
                    " See ooniprobe.conf privacy.includepcap")
            sys.exit(2)
    global_options['check_incoherences'] = check_incoherences
    return global_options

def setupAnnotations(global_options):
    annotations={}
    for annotation in global_options["annotations"].split(","):
        pair = annotation.split(":")
        if len(pair) == 2:
            key = pair[0].strip()
            value = pair[1].strip()
            annotations[key] = value
        else:
            log.err("Invalid annotation: %s" % annotation)
            sys.exit(1)
    global_options["annotations"] = annotations
    return annotations

def setupCollector(global_options, collector_client):
    from backend_client import CollectorClient

    if global_options['collector']:
        collector_client = CollectorClient(global_options['collector'])
    elif config.reports.get('collector', None) is not None:
        collector_client = CollectorClient(config.reports['collector'])
    if not collector_client.isSupported():
        raise errors.CollectorUnsupported
    return collector_client

def createDeck(global_options, url=None):
    from ooni.nettest import NetTestLoader
    from ooni.deck import Deck, nettest_to_path
    from backend_client import CollectorClient

    if url:
        log.msg("Creating deck for: %s" % (url))

    if global_options['no-yamloo']:
        log.msg("Will not write to a yamloo report file")

    deck = Deck(bouncer=global_options['bouncer'],
                no_collector=global_options['no-collector'])

    try:
        if global_options['testdeck']:
            deck.loadDeck(global_options['testdeck'], global_options)
        else:
            log.debug("No test deck detected")
            test_file = nettest_to_path(global_options['test_file'], True)
            if url is not None:
                args = ('-u', url)
            else:
                args = tuple()
            if any(global_options['subargs']):
                args = global_options['subargs'] + args
            net_test_loader = NetTestLoader(args,
                                            test_file=test_file,
                                            annotations=global_options['annotations'])
            if global_options['collector']:
                net_test_loader.collector = \
                    CollectorClient(global_options['collector'])
            deck.insert(net_test_loader)
    except errors.MissingRequiredOption as option_name:
        log.err('Missing required option: "%s"' % option_name)
        incomplete_net_test_loader = option_name.net_test_loader
        print incomplete_net_test_loader.usageOptions().getUsage()
        sys.exit(2)
    except errors.NetTestNotFound as path:
        log.err('Requested NetTest file not found (%s)' % path)
        sys.exit(3)
    except errors.OONIUsageError as e:
        log.err(e)
        print e.net_test_loader.usageOptions().getUsage()
        sys.exit(4)
    except errors.HTTPSCollectorUnsupported:
        log.err("HTTPS collectors require a twisted version of at least 14.0.2.")
        sys.exit(6)
    except errors.InsecureBackend:
        log.err("Attempting to report to an insecure collector.")
        log.err("To enable reporting to insecure collector set the "
                "advanced->insecure_backend option to true in "
                "your ooniprobe.conf file.")
        sys.exit(7)
    except Exception as e:
        if config.advanced.debug:
            log.exception(e)
        log.err(e)
        sys.exit(5)

    return deck


def runTestWithDirector(director, global_options, url=None, start_tor=True):
    deck = createDeck(global_options, url=url)

    start_tor |= deck.requiresTor

    d = director.start(start_tor=start_tor,
                       check_incoherences=global_options['check_incoherences'])

    def setup_nettest(_):
        try:
            return deck.setup()
        except errors.UnableToLoadDeckInput as error:
            return defer.failure.Failure(error)
        except errors.NoReachableTestHelpers as error:
            return defer.failure.Failure(error)
        except errors.NoReachableCollectors as error:
            return defer.failure.Failure(error)

    # Wait until director has started up (including bootstrapping Tor)
    # before adding tests
    @defer.inlineCallbacks
    def post_director_start(_):
        for net_test_loader in deck.netTestLoaders:
            # Decks can specify different collectors
            # for each net test, so that each NetTest
            # may be paired with a test_helper and its collector
            # However, a user can override this behavior by
            # specifying a collector from the command-line (-c).
            # If a collector is not specified in the deck, or the
            # deck is a singleton, the default collector set in
            # ooniprobe.conf will be used
            collector_client = None
            if not global_options['no-collector']:
                collector_client = setupCollector(global_options,
                                                  net_test_loader.collector)

            yield director.startNetTest(net_test_loader,
                                        global_options['reportfile'],
                                        collector_client,
                                        global_options['no-yamloo'])

    d.addCallback(setup_nettest)
    d.addCallback(post_director_start)
    d.addErrback(director_startup_handled_failures)
    d.addErrback(director_startup_other_failures)
    return d

def runWithDirector(global_options):
    """
    Instance the director, parse command line options and start an ooniprobe
    test!
    """
    from ooni.director import Director
    start_tor = False
    director = Director()
    if global_options['list']:
        net_tests = [net_test for net_test in director.getNetTests().items()]
        print ""
        print "Installed nettests"
        print "=================="
        for net_test_id, net_test in net_tests:
            optList = []
            for name, details in net_test['arguments'].items():
                optList.append({'long': name, 'doc': details['description']})

            desc = ('\n' +
                    net_test['name'] +
                    '\n' +
                    '-'*len(net_test['name']) +
                    '\n' +
                    '\n'.join(textwrap.wrap(net_test['description'], 80)) +
                    '\n\n' +
                    '$ ooniprobe {}/{}'.format(net_test['category'],
                                                      net_test['id']) +
                    '\n\n' +
                    ''.join(usage.docMakeChunks(optList))
            )
            print desc
            print "Note: Third party tests require an external "\
                  "application to run properly."

        sys.exit(0)

    elif global_options['printdeck']:
        del global_options['printdeck']
        print "# Copy and paste the lines below into a test deck to run the specified test with the specified arguments"
        print yaml.safe_dump([{'options': global_options}]).strip()

        sys.exit(0)

    if global_options.get('annotations') is not None:
        global_options['annotations'] = setupAnnotations(global_options)

    if global_options.get('preferred-backend') is not None:
        config.advanced.preferred_backend = global_options['preferred-backend']

    if global_options['no-collector']:
        log.msg("Not reporting using a collector")
        global_options['collector'] = None
        start_tor = False
    elif config.advanced.get("preferred_backend", "onion") == "onion":
        start_tor = True

    if (global_options['collector'] and
            config.advanced.get("preferred_backend", "onion") == "onion"):
        start_tor |= True

    return runTestWithDirector(director=director,
                               start_tor=start_tor,
                               global_options=global_options)


# this variant version of runWithDirector splits the process in two,
# allowing a single director instance to be reused with multiple decks.

def runWithDaemonDirector(global_options):
    """
    Instance the director, parse command line options and start an ooniprobe
    test!
    """
    from twisted.internet import reactor, protocol
    from ooni.director import Director
    try:
        import pika
        from pika import exceptions
        from pika.adapters import twisted_connection
    except ImportError:
        print "Pika is required for queue connection."
        print "Install with \"pip install pika\"."
        sys.exit(7)

    director = Director()

    if global_options.get('annotations') is not None:
        global_options['annotations'] = setupAnnotations(global_options)

    if global_options['no-collector']:
        log.msg("Not reporting using a collector")
        global_options['collector'] = None
        start_tor = False
    else:
        start_tor = True

    finished = defer.Deferred()

    @defer.inlineCallbacks
    def readmsg(_, channel, queue_object, consumer_tag, counter):

        # Wait for a message and decode it.
        if counter >= lifetime:
            log.msg("Counter")
            queue_object.close(LifetimeExceeded())
            yield channel.basic_cancel(consumer_tag=consumer_tag)
            finished.callback(None)

        else:
            log.msg("Waiting for message")

            try:
                ch, method, properties, body = yield queue_object.get()
                log.msg("Got message")
                data = json.loads(body)
                counter += 1

                log.msg("Received %d/%d: %s" % (counter, lifetime, data['url'],))
                # acknowledge the message
                ch.basic_ack(delivery_tag=method.delivery_tag)

                d = runTestWithDirector(director=director,
                                        start_tor=start_tor,
                                        global_options=global_options,
                                        url=data['url'].encode('utf8'))
                # When the test has been completed, go back to waiting for a message.
                d.addCallback(readmsg, channel, queue_object, consumer_tag, counter+1)
            except exceptions.AMQPError,v:
                log.msg("Error")
                log.exception(v)
                finished.errback(v)



    @defer.inlineCallbacks
    def runQueue(connection, name, qos):
        # Set up the queue consumer.  When a message is received, run readmsg
        channel = yield connection.channel()
        yield channel.basic_qos(prefetch_count=qos)
        queue_object, consumer_tag = yield channel.basic_consume(
                                                   queue=name,
                                                   no_ack=False)
        readmsg(None, channel, queue_object, consumer_tag, 0)



    # Create the AMQP connection.  This could be refactored to allow test URLs
    # to be submitted through an HTTP server interface or something.
    urlp = urlparse.urlparse(config.global_options['queue'])
    urlargs = dict(urlparse.parse_qsl(urlp.query))

    # random lifetime requests counter
    lifetime = random.randint(820, 1032)

    # AMQP connection details are sent through the cmdline parameter '-Q'
    creds = pika.PlainCredentials(urlp.username or 'guest',
                                  urlp.password or 'guest')
    parameters = pika.ConnectionParameters(urlp.hostname,
                                           urlp.port or 5672,
                                           urlp.path.rsplit('/',1)[0] or '/',
                                           creds,
                                           heartbeat_interval=120,
                                           )
    cc = protocol.ClientCreator(reactor,
                                twisted_connection.TwistedProtocolConnection,
                                parameters)
    d = cc.connectTCP(urlp.hostname, urlp.port or 5672)
    d.addCallback(lambda protocol: protocol.ready)
    # start the wait/process sequence.
    d.addCallback(runQueue, urlp.path.rsplit('/',1)[-1], int(urlargs.get('qos',1)))

    return finished



import os
import tarfile
import tempfile

from twisted.python.filepath import FilePath
from twisted.internet import defer
from twisted.web.client import downloadPage

from ooni.settings import config
from ooni.resources import ooni_resources_url

@defer.inlineCallbacks
def download_resources():
    if os.access(config.var_lib_path, os.W_OK):
        dst_directory = FilePath(config.var_lib_path)
    else:
        dst_directory = FilePath(config.ooni_home)

    print("Downloading {} to {}".format(ooni_resources_url,
                                        dst_directory.path))
    tmp_download_directory = FilePath(tempfile.mkdtemp())
    tmp_download_filename = tmp_download_directory.temporarySibling()


    try:
        yield downloadPage(ooni_resources_url, tmp_download_filename.path)
        ooni_resources_tar_gz = tarfile.open(tmp_download_filename.path)
        ooni_resources_tar_gz.extractall(tmp_download_directory.path)

        if not tmp_download_directory.child('GeoIP').exists():
            raise Exception("Could not find GeoIP data files in downloaded "
                            "tar.")

        if not tmp_download_directory.child('resources').exists():
            raise Exception("Could not find resources data files in "
                            "downloaded tar.")

        geoip_dir = dst_directory.child('GeoIP')
        resources_dir = dst_directory.child('resources')

        if geoip_dir.exists():
            geoip_dir.remove()
        tmp_download_directory.child('GeoIP').moveTo(geoip_dir)

        if resources_dir.exists():
            resources_dir.remove()
        tmp_download_directory.child('resources').moveTo(resources_dir)

        print("Written GeoIP files to {}".format(geoip_dir.path))
        print("Written resources files to {}".format(resources_dir.path))

    except Exception as exc:
        print("Failed to download resources!")
        raise exc

    finally:
        tmp_download_directory.remove()



import os
import distutils.spawn

from twisted.internet import defer, reactor
from twisted.internet.endpoints import TCP4ClientEndpoint
from twisted.web.client import ProxyAgent, readBody
from twisted.python import usage

from ooni.templates.process import ProcessTest, ProcessDirector
from ooni.utils import log, net
from ooni.errors import handleAllFailures

class LanternNotInstalled(Exception):
    pass

class UsageOptions(usage.Options):
    optParameters = [
        ['url', 'u', net.GOOGLE_HUMANS[0],
            'Specify the URL to fetch over lantern (default: http://www.google.com/humans.txt).'],
        ['expected-body', 'e', net.GOOGLE_HUMANS[1],
            'Specify the beginning of the expected body in the response (default: ' + net.GOOGLE_HUMANS[1] + ').']
    ]

class LanternTest(ProcessTest):
    """
    This class tests Lantern (https://getlantern.org).

    test_lantern_circumvent
      Starts Lantern on Linux in --headless mode and
      determine if it bootstraps successfully or not.
      Then, make a HTTP request for http://google.com
      and records the response body or failure string.

    """

    name = "Lantern Circumvention Tool Test"
    description = "Bootstraps Lantern, connects to a URL and verifies if it "\
                  "contains the expected input."
    author = "Aaron Gibson"
    version = "0.1.0"
    timeout = 120
    usageOptions = UsageOptions

    def requirements(self):
        if not distutils.spawn.find_executable("lantern"):
            raise LanternNotInstalled('lantern is not installed')

    def setUp(self):
        self.report['body'] = None
        self.report['failure'] = None
        self.report['success'] = None
        self.report['default_configuration'] = True

        self.command = [distutils.spawn.find_executable("lantern"), "--headless"]
        self.bootstrapped = defer.Deferred()
        self.exited = False

        self.url = self.localOptions['url']
        if self.url != net.GOOGLE_HUMANS[0]:
            self.report['default_configuration'] = False

        if self.localOptions['expected-body'] != net.GOOGLE_HUMANS[1]:
            self.report['default_configuration'] = False

    def stop(self, reason=None):
        if not self.exited:
            self.processDirector.close()
            self.processDirector.transport.signalProcess('TERM')
            self.exited = True

    def handleRead(self, stdout=None, stderr=None):
        """
        This is called with each chunk of data from stdout and stderr.
        """
        if not self.bootstrapped.called and "Successfully dialed via" in self.processDirector.stdout:
            log.msg("Lantern connection successful")
            self.processDirector.cancelTimer()
            self.bootstrapped.callback("bootstrapped")

    def test_lantern_circumvent(self):
        def addResultToReport(result):
            self.report['body'] = result
            if result.startswith(self.localOptions['expected-body']):
                log.msg("Got the HTTP response body I expected!")
                self.report['success'] = True
            else:
                self.report['success'] = False

        def addFailureToReport(failure):
            log.err("Failed to connect to lantern")
            log.failure(failure)
            self.report['failure'] = handleAllFailures(failure)
            self.report['success'] = False

        def doRequest(noreason):
            proxyEndpoint = TCP4ClientEndpoint(reactor, '127.0.0.1', 8787)
            agent = ProxyAgent(proxyEndpoint, reactor)
            log.msg("Doing HTTP request via Lantern (127.0.0.1:8787) for %s" % self.url)
            request = agent.request("GET", self.url)
            request.addCallback(readBody)
            request.addCallback(addResultToReport)
            request.addCallback(self.processDirector.close)
            return request

        self.bootstrapped.addCallback(doRequest)
        self.bootstrapped.addErrback(addFailureToReport)
        self.bootstrapped.addBoth(self.stop)
        self.d = self.run(self.command, env=os.environ, usePTY=1)
        return self.d



import yaml


class ReportLoader(object):
    _header_keys = (
        'probe_asn',
        'probe_cc',
        'probe_ip',
        'start_time',
        'test_name',
        'test_version',
        'options',
        'input_hashes',
        'software_name',
        'software_version'
    )

    def __init__(self, report_filename):
        self._fp = open(report_filename)
        self._yfp = yaml.safe_load_all(self._fp)

        self.header = self._yfp.next()

    def __iter__(self):
        return self

    def next(self):
        try:
            return self._yfp.next()
        except StopIteration:
            self.close()
            raise StopIteration

    def close(self):
        self._fp.close()



from base64 import b64encode
from ooni.nettest import NetTestCase
from ooni.utils import log
from ooni.settings import config
from ooni.utils.net import hasRawSocketPermission

from ooni.utils.txscapy import ScapySender, ScapyFactory


def _representPacket(packet):
    return {
        "raw_packet": {
            'data': b64encode(str(packet)),
            'format': 'base64'
        },
        "summary": repr(packet)
    }

class BaseScapyTest(NetTestCase):

    """
    The report of a test run with scapy looks like this:

    report:
        sent_packets: [
            {
            'raw_packet': BASE64Encoding of packet,
            'summary': 'IP / TCP 192.168.2.66:ftp_data > 8.8.8.8:http S'
            }
        ]

        answered_packets: []

    """
    name = "Base Scapy Test"
    version = 0.1

    requiresRoot = not hasRawSocketPermission()
    baseFlags = [
        ['ipsrc', 's',
         'Does *not* check if IP src and ICMP IP citation '
         'matches when processing answers'],
        ['seqack', 'k',
         'Check if TCP sequence number and ACK match in the '
         'ICMP citation when processing answers'],
        ['ipid', 'i', 'Check if the IPID matches when processing answers']]

    def _setUp(self):
        super(BaseScapyTest, self)._setUp()

        if config.scapyFactory is None:
            log.debug("Scapy factory not set, registering it.")
            config.scapyFactory = ScapyFactory(config.advanced.interface)

        self.report['answer_flags'] = []
        if self.localOptions['ipsrc']:
            config.checkIPsrc = 0
        else:
            self.report['answer_flags'].append('ipsrc')
            config.checkIPsrc = 1

        if self.localOptions['ipid']:
            self.report['answer_flags'].append('ipid')
            config.checkIPID = 1
        else:
            config.checkIPID = 0
        # XXX we don't support strict matching
        # since (from scapy's documentation), some stacks have a bug for which
        # the bytes in the IPID are swapped.
        # Perhaps in the future we will want to have more fine grained control
        # over this.

        if self.localOptions['seqack']:
            self.report['answer_flags'].append('seqack')
            config.check_TCPerror_seqack = 1
        else:
            config.check_TCPerror_seqack = 0

        self.report['sent_packets'] = []
        self.report['answered_packets'] = []

    def finishedSendReceive(self, packets):
        """
        This gets called when all packets have been sent and received.
        """
        answered, unanswered = packets

        for snd, rcv in answered:
            log.debug("Writing report for scapy test")
            sent_packet = snd
            received_packet = rcv

            if not config.privacy.includeip:
                log.debug("Detected you would not like to "
                          "include your ip in the report")
                log.debug(
                    "Stripping source and destination IPs from the reports")
                sent_packet.src = '127.0.0.1'
                received_packet.dst = '127.0.0.1'

            self.report['sent_packets'].append(_representPacket(sent_packet))
            self.report['answered_packets'].append(_representPacket(received_packet))
        return packets

    def sr(self, packets, timeout=None, *arg, **kw):
        """
        Wrapper around scapy.sendrecv.sr for sending and receiving of packets
        at layer 3.
        """
        scapySender = ScapySender(timeout=timeout)

        config.scapyFactory.registerProtocol(scapySender)
        log.debug("Using sending with hash %s" % scapySender.__hash__)

        d = scapySender.startSending(packets)
        d.addCallback(self.finishedSendReceive)
        return d

    def sr1(self, packets, *arg, **kw):
        def done(packets):
            """
            We do this so that the returned value is only the one packet that
            we expected a response for, identical to the scapy implementation
            of sr1.
            """
            try:
                return packets[0][0][1]
            except IndexError:
                log.err("Got no response...")
                return packets

        scapySender = ScapySender()
        scapySender.expected_answers = 1

        config.scapyFactory.registerProtocol(scapySender)

        log.debug("Running sr1")
        d = scapySender.startSending(packets)
        log.debug("Started to send")
        d.addCallback(self.finishedSendReceive)
        d.addCallback(done)
        return d

    def send(self, packets, *arg, **kw):
        """
        Wrapper around scapy.sendrecv.send for sending of packets at layer 3
        """
        scapySender = ScapySender()

        config.scapyFactory.registerProtocol(scapySender)
        scapySender.startSending(packets)

        scapySender.stopSending()
        for sent_packet in packets:
            self.report['sent_packets'].append(_representPacket(sent_packet))


ScapyTest = BaseScapyTest
